@@ -19,7 +19,10 @@
 #ifndef UNITY_SCOPES_INTERNAL_ACTIVATIONQUERYBASEIMPL_H
 #define UNITY_SCOPES_INTERNAL_ACTIVATIONQUERYBASEIMPL_H
 
+#include <unity/scopes/internal/QueryBaseImpl.h>
 #include <unity/scopes/ActivationResponse.h>
+#include <unity/scopes/Result.h>
+#include <unity/scopes/ActionMetadata.h>
 #include <unity/util/NonCopyable.h>
 
 namespace unity
@@ -31,14 +34,30 @@ namespace scopes
 namespace internal
 {
 
-class ActivationQueryBaseImpl final
+class ActivationQueryBaseImpl : public QueryBaseImpl
 {
 public:
     NONCOPYABLE(ActivationQueryBaseImpl);
 
-    ActivationQueryBaseImpl() = default;
+    ActivationQueryBaseImpl(Result const& result, ActionMetadata const& metadata, std::string const& widget_id = "", std::string const& action_id = "");
     ~ActivationQueryBaseImpl() = default;
     ActivationResponse activate();
+
+    Result result() const;
+    ActionMetadata action_metadata() const;
+    std::string widget_id() const;
+    std::string action_id() const;
+
+    void cancel() override;
+    bool valid() const override;
+
+private:
+    Result result_;
+    const ActionMetadata metadata_;
+    const std::string widget_id_;
+    const std::string action_id_;
+    bool valid_;
+    mutable std::mutex mutex_;
 };
 
 } // namespace internal
