@@ -20,6 +20,7 @@
 #define UNITY_SCOPES_INTERNAL_MWREGISTRY_H
 
 #include <unity/scopes/internal/MWObjectProxy.h>
+#include <unity/scopes/internal/MWSubscriber.h>
 #include <unity/scopes/Registry.h>
 #include <unity/scopes/ScopeMetadata.h>
 
@@ -38,12 +39,23 @@ public:
     // Remote operations
     virtual ScopeMetadata get_metadata(std::string const& scope_id) = 0;
     virtual MetadataMap list() = 0;
+    virtual ObjectProxy locate(std::string const& identity, int64_t timeout) = 0;
     virtual ObjectProxy locate(std::string const& identity) = 0;
+    virtual bool is_scope_running(std::string const& scope_id) = 0;
 
     virtual ~MWRegistry();
 
+    // Local operations
+    core::ScopedConnection set_scope_state_callback(std::string const& scope_id, std::function<void(bool)> callback);
+    core::ScopedConnection set_list_update_callback(std::function<void()> callback);
+
 protected:
     MWRegistry(MiddlewareBase* mw_base);
+
+private:
+    MiddlewareBase* mw_base_;
+    MWSubscriber::UPtr list_update_subscriber_;
+    std::map<std::string, MWSubscriber::UPtr> scope_state_subscribers_;
 };
 
 } // namespace internal
