@@ -21,6 +21,8 @@
 
 #include <unity/scopes/PreviewReply.h>
 
+#include <unity/scopes/testing/MockObject.h>
+
 #include <gmock/gmock.h>
 
 namespace unity
@@ -34,7 +36,7 @@ namespace testing
 
 /// @cond
 
-class MockPreviewReply : public unity::scopes::PreviewReply
+class MockPreviewReply : public unity::scopes::PreviewReply, public unity::scopes::testing::MockObject
 {
 public:
     MockPreviewReply() = default;
@@ -42,6 +44,7 @@ public:
     // From Reply
     MOCK_METHOD0(finished, void());
     MOCK_METHOD1(error, void(std::exception_ptr));
+    MOCK_METHOD1(info, void(OperationInfo const&));
 
     // From SearchReply
     MOCK_METHOD1(register_layout, bool(ColumnLayoutList const&));
