@@ -43,6 +43,15 @@ public:
     MOCK_METHOD1(get_metadata, ScopeMetadata(std::string const&));
     MOCK_METHOD0(list, MetadataMap());
     MOCK_METHOD1(list_if, MetadataMap(std::function<bool(ScopeMetadata const&)>));
+    MOCK_METHOD1(is_scope_running, bool(std::string const&));
+    core::ScopedConnection set_scope_state_callback(std::string const&, std::function<void(bool is_running)>) override
+    {
+        return core::Signal<>().connect([]{});
+    }
+    core::ScopedConnection set_list_update_callback(std::function<void()>) override
+    {
+        return core::Signal<>().connect([]{});
+    }
 };
 
 /// @endcond
