@@ -18,6 +18,8 @@
 
 #include <unity/scopes/internal/MiddlewareBase.h>
 
+#include <unity/UnityExceptions.h>
+
 #include <cassert>
 
 using namespace std;
@@ -34,7 +36,6 @@ namespace internal
 MiddlewareBase::MiddlewareBase(RuntimeImpl* runtime) :
     runtime_(runtime)
 {
-    assert(runtime);
 }
 
 MiddlewareBase::~MiddlewareBase()
@@ -43,6 +44,12 @@ MiddlewareBase::~MiddlewareBase()
 
 RuntimeImpl* MiddlewareBase::runtime() const noexcept
 {
+    if (!runtime_)
+    {
+        // This is the case only for some of the tests, because it impossible to
+        // fake RuntimeImpl.
+        throw LogicException("MiddlewareBase::runtime(): MiddlewareBase was instantianted with nullptr runtime");
+    }
     return runtime_;
 }
 
