@@ -14,26 +14,43 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Authored by: Marcus Tomlinson <marcus.tomlinson@canonical.com>
+ *              Pawel Stolowski <pawel.stolowski@canonical.com>
  */
 
+#include <unity/scopes/internal/FilterBaseImpl.h>
+#include <unity/scopes/internal/FilterStateImpl.h>
 #include <unity/scopes/internal/smartscopes/SmartScopesClient.h>
+
+#include <unity/scopes/ScopeExceptions.h>
 #include <unity/UnityExceptions.h>
 
 #include <algorithm>
+#include <array>
 #include <cstring>
 #include <fstream>
 #include <future>
+#include <utility>
 #include <iostream>
 #include <map>
 #include <sstream>
 #include <sys/stat.h>
 
+static std::string homedir()
+{
+    static const char* home = getenv("HOME");
+    if (!home)
+    {
+        throw unity::scopes::ConfigException("smartscopesclient: HOME not set, cannot configure cache");
+    }
+    return home;
+}
+
 static const std::string c_base_url = "https://dash.ubuntu.com/smartscopes/v2";
 static const std::string c_remote_scopes_resource = "/remote-scopes";
 static const std::string c_search_resource = "/search";
 static const std::string c_preview_resource = "/preview";
 
-static const std::string c_scopes_cache_dir = std::string(getenv("HOME")) + "/.cache/unity-scopes/";
+static const std::string c_scopes_cache_dir = homedir() + "/.cache/unity-scopes/";
 static const std::string c_scopes_cache_filename = "remote-scopes.json";
 
 using namespace unity::scopes;
@@ -52,9 +69,9 @@ SearchHandle::~SearchHandle()
     cancel_search();
 }
 
-std::vector<SearchResult> SearchHandle::get_search_results()
+void SearchHandle::wait()
 {
-    return ssc_->get_search_results(search_id_);
+    ssc_->wait_for_search(search_id_);
 }
 
 void SearchHandle::cancel_search()
@@ -75,9 +92,9 @@ PreviewHandle::~PreviewHandle()
     cancel_preview();
 }
 
-std::pair<PreviewHandle::Columns, PreviewHandle::Widgets> PreviewHandle::get_preview_results()
+void PreviewHandle::wait()
 {
-    return ssc_->get_preview_results(preview_id_);
+    ssc_->wait_for_preview(preview_id_);
 }
 
 void PreviewHandle::cancel_preview()
@@ -146,14 +163,16 @@ bool SmartScopesClient::get_remote_scopes(std::vector<RemoteScope>& remote_scope
 
         if (!locale.empty())
         {
-            remote_scopes_uri << "&locale=\"" << locale << "\"";
+            remote_scopes_uri << "locale=" << locale;
         }
 
         std::cout << "SmartScopesClient.get_remote_scopes(): GET " << remote_scopes_uri.str() << std::endl;
-        HttpResponseHandle::SPtr response = http_client_->get(remote_scopes_uri.str());
+
+        HttpResponseHandle::SPtr response = http_client_->get(remote_scopes_uri.str(), [&response_str](std::string const& replyLine) {
+                response_str += replyLine; // accumulate all reply lines
+        });
         response->wait();
 
-        response_str = response->get();
         std::cout << "SmartScopesClient.get_remote_scopes(): Remote scopes:" << std::endl << response_str << std::endl;
     }
     catch (std::exception const& e)
@@ -196,6 +215,7 @@ bool SmartScopesClient::get_remote_scopes(std::vector<RemoteScope>& remote_scope
         throw;
     }
 
+    remote_scopes.clear();
     for (int i = 0; i < root_node->size(); ++i)
     {
         RemoteScope scope;
@@ -230,7 +250,6 @@ bool SmartScopesClient::get_remote_scopes(std::vector<RemoteScope>& remote_scope
                 continue;
             }
 
-            scope.id = child_node->get_node("id")->as_string();
             scope.name = child_node->get_node("name")->as_string();
             scope.description = child_node->get_node("description")->as_string();
             scope.author = child_node->get_node("author")->as_string();
@@ -246,6 +265,21 @@ bool SmartScopesClient::get_remote_scopes(std::vector<RemoteScope>& remote_scope
                 scope.art.reset(new std::string(child_node->get_node("art")->as_string()));
             }
 
+            if (child_node->has_node("appearance"))
+            {
+                scope.appearance.reset(new VariantMap(child_node->get_node("appearance")->to_variant().get_dict()));
+            }
+
+            if (child_node->has_node("settings"))
+            {
+                scope.settings.reset(new std::string(child_node->get_node("settings")->to_json_string()));
+            }
+
+            if (child_node->has_node("needs_location_data"))
+            {
+                scope.needs_location_data.reset(new bool(child_node->get_node("needs_location_data")->as_bool()));
+            }
+
             scope.invisible = child_node->has_node("invisible") ? child_node->get_node("invisible")->as_bool() : false;
 
             remote_scopes.push_back(scope);
@@ -290,14 +324,18 @@ bool SmartScopesClient::get_remote_scopes(std::vector<RemoteScope>& remote_scope
     return !using_cache;
 }
 
-SearchHandle::UPtr SmartScopesClient::search(std::string const& base_url,
+SearchHandle::UPtr SmartScopesClient::search(SearchReplyHandler const& handler,
+                                             std::string const& base_url,
                                              std::string const& query,
                                              std::string const& department_id,
                                              std::string const& session_id,
-                                             uint query_id,
+                                             int query_id,
                                              std::string const& platform,
+                                             VariantMap const& settings,
+                                             VariantMap const& filter_state,
                                              std::string const& locale,
                                              std::string const& country,
+                                             std::string const& user_agent_hdr,
                                              uint limit)
 {
     std::ostringstream search_uri;
@@ -306,7 +344,7 @@ SearchHandle::UPtr SmartScopesClient::search(std::string const& base_url,
     // mandatory parameters
     search_uri << "q=" << http_client_->to_percent_encoding(query);
 
-    search_uri << "&session_id=" << session_id;
+    search_uri << "&session_id=" << http_client_->to_percent_encoding(session_id);
     search_uri << "&query_id=" << std::to_string(query_id);
     search_uri << "&platform=" << platform;
 
@@ -315,6 +353,14 @@ SearchHandle::UPtr SmartScopesClient::search(std::string const& base_url,
     {
         search_uri << "&department=" << http_client_->to_percent_encoding(department_id);
     }
+    if (!settings.empty())
+    {
+        std::string settings_str = stringify_settings(settings);
+        if (!settings_str.empty())
+        {
+            search_uri << "&settings=" << http_client_->to_percent_encoding(settings_str);
+        }
+    }
     if (!locale.empty())
     {
         search_uri << "&locale=" << locale;
@@ -327,22 +373,46 @@ SearchHandle::UPtr SmartScopesClient::search(std::string const& base_url,
     {
         search_uri << "&limit=" << std::to_string(limit);
     }
+    if (!filter_state.empty())
+    {
+        search_uri << "&filters=" << http_client_->to_percent_encoding(Variant(filter_state).serialize_json());
+    }
 
     std::lock_guard<std::mutex> lock(query_results_mutex_);
     uint search_id = ++query_counter_;
 
     std::cout << "SmartScopesClient.search(): GET " << search_uri.str() << std::endl;
-    query_results_[search_id] = http_client_->get(search_uri.str());
+
+    HttpHeaders headers;
+    if (!user_agent_hdr.empty())
+    {
+        std::cout << "User agent: " << user_agent_hdr;
+        headers.push_back(std::make_pair("User-Agent", user_agent_hdr));
+    }
+
+    query_results_[search_id] = http_client_->get(search_uri.str(), [this, handler](std::string const& lineData) {
+            try
+            {
+                parse_line(lineData, handler);
+            }
+            catch (std::exception const &e)
+            {
+                std::cerr << "Failed to parse: " << e.what() << std::endl;
+            }
+    }, headers);
 
     return SearchHandle::UPtr(new SearchHandle(search_id, shared_from_this()));
 }
 
-PreviewHandle::UPtr SmartScopesClient::preview(std::string const& base_url,
+PreviewHandle::UPtr SmartScopesClient::preview(PreviewReplyHandler const& handler,
+                                               std::string const& base_url,
                                                std::string const& result,
                                                std::string const& session_id,
                                                std::string const& platform,
                                                const uint widgets_api_version,
+                                               VariantMap const& settings,
                                                std::string const& locale,
+                                               std::string const& user_agent_hdr,
                                                std::string const& country)
 {
     std::ostringstream preview_uri;
@@ -351,12 +421,26 @@ PreviewHandle::UPtr SmartScopesClient::preview(std::string const& base_url,
     // mandatory parameters
 
     preview_uri << "result=" << http_client_->to_percent_encoding(result);
-    preview_uri << "&session_id=" << session_id;
+    preview_uri << "&session_id=" << http_client_->to_percent_encoding(session_id);
     preview_uri << "&platform=" << platform;
     preview_uri << "&widgets_api_version=" << std::to_string(widgets_api_version);
 
+    HttpHeaders headers;
+    if (!user_agent_hdr.empty())
+    {
+        headers.push_back(std::make_pair("User-Agent", user_agent_hdr));
+    }
+
     // optional parameters
 
+    if (!settings.empty())
+    {
+        std::string settings_str = stringify_settings(settings);
+        if (!settings_str.empty())
+        {
+            preview_uri << "&settings=" << http_client_->to_percent_encoding(settings_str);
+        }
+    }
     if (!locale.empty())
     {
         preview_uri << "&locale=" << locale;
@@ -370,18 +454,153 @@ PreviewHandle::UPtr SmartScopesClient::preview(std::string const& base_url,
     uint preview_id = ++query_counter_;
 
     std::cout << "SmartScopesClient.preview(): GET " << preview_uri.str() << std::endl;
-    query_results_[preview_id] = http_client_->get(preview_uri.str());
+    query_results_[preview_id] = http_client_->get(preview_uri.str(), [this, handler](std::string const& lineData) {
+            try
+            {
+                parse_line(lineData, handler);
+            }
+            catch (std::exception const &e)
+            {
+                std::cerr << "Failed to parse: " << e.what() << std::endl;
+            }
+    }, headers);
 
     return PreviewHandle::UPtr(new PreviewHandle(preview_id, shared_from_this()));
 }
 
-std::vector<SearchResult> SmartScopesClient::get_search_results(uint search_id)
+void SmartScopesClient::parse_line(std::string const& json, PreviewReplyHandler const& handler)
+{
+    JsonNodeInterface::SPtr root_node;
+    JsonNodeInterface::SPtr child_node;
+    {
+        std::lock_guard<std::mutex> lock(json_node_mutex_);
+        json_node_->read_json(json);
+        root_node = json_node_->get_node();
+    }
+
+    if (root_node->has_node("columns"))
+    {
+        PreviewHandle::Columns columns;
+        child_node = root_node->get_node("columns");
+
+        // for each column
+        for (int column_i = 0; column_i < child_node->size(); ++column_i)
+        {
+            auto column_node = child_node->get_node(column_i);
+
+            // for each widget layout within the column
+            std::vector<std::vector<std::string>> widget_layouts;
+            for (int widget_lo_i = 0; widget_lo_i < column_node->size(); ++widget_lo_i)
+            {
+                auto widget_lo_node = column_node->get_node(widget_lo_i);
+
+                // for each widget within the widget layout
+                std::vector<std::string> widget_ids;
+                for (int widget_i = 0; widget_i < widget_lo_node->size(); ++widget_i)
+                {
+                    auto widget_node = widget_lo_node->get_node(widget_i);
+                    widget_ids.push_back(widget_node->as_string());
+                }
+
+                widget_layouts.push_back(widget_ids);
+            }
+
+            columns.push_back(widget_layouts);
+        }
+        handler.columns_handler(columns);
+    }
+    else if (root_node->has_node("widget"))
+    {
+        child_node = root_node->get_node("widget");
+        handler.widget_handler(child_node->to_json_string());
+    }
+}
+
+void SmartScopesClient::parse_line(std::string const& json, SearchReplyHandler const& handler)
+{
+    JsonNodeInterface::SPtr root_node;
+    JsonNodeInterface::SPtr child_node;
+
+    {
+        std::lock_guard<std::mutex> lock(json_node_mutex_);
+        json_node_->read_json(json);
+        root_node = json_node_->get_node();
+    }
+
+    if (root_node->has_node("category"))
+    {
+        child_node = root_node->get_node("category");
+        auto category = std::make_shared<SearchCategory>();
+
+        std::vector<std::string> members = child_node->member_names();
+        for (auto& member : members)
+        {
+            if (member == "icon")
+            {
+                category->icon = child_node->get_node(member)->as_string();
+            }
+            else if (member == "id")
+            {
+                category->id = child_node->get_node(member)->as_string();
+            }
+            else if (member == "render_template")
+            {
+                category->renderer_template = child_node->get_node(member)->as_string();
+            }
+            else if (member == "title")
+            {
+                category->title = child_node->get_node(member)->as_string();
+            }
+        }
+        handler.category_handler(category);
+    }
+    else if (root_node->has_node("result"))
+    {
+        child_node = root_node->get_node("result");
+        SearchResult result;
+        result.json = child_node->to_json_string();
+
+        std::vector<std::string> members = child_node->member_names();
+        for (auto& member : members)
+        {
+            if (member == "uri")
+            {
+                result.uri = child_node->get_node(member)->as_string();
+            }
+            else if (member == "cat_id")
+            {
+                std::string category = child_node->get_node(member)->as_string();
+                result.category_id = category;
+            }
+            else
+            {
+                result.other_params[member] = child_node->get_node(member);
+            }
+        }
+        handler.result_handler(result);
+    }
+    else if (root_node->has_node("departments"))
+    {
+        auto departments = parse_departments(root_node->get_node("departments"));
+        handler.departments_handler(departments);
+    }
+    else if (root_node->has_node("filters"))
+    {
+        auto filters = parse_filters(root_node->get_node("filters"));
+        handler.filters_handler(filters);
+    }
+    else if (root_node->has_node("filter_state"))
+    {
+        auto filter_state = parse_filter_state(root_node->get_node("filter_state"));
+        handler.filter_state_handler(filter_state);
+    }
+}
+
+void SmartScopesClient::wait_for_search(uint search_id)
 {
     try
     {
         HttpResponseHandle::SPtr query_result;
-        std::string response_str;
-
         {
             std::lock_guard<std::mutex> lock(query_results_mutex_);
 
@@ -395,186 +614,118 @@ std::vector<SearchResult> SmartScopesClient::get_search_results(uint search_id)
         }
 
         query_result->wait();
-        response_str = query_result->get();
-
-        {
-            std::lock_guard<std::mutex> lock(query_results_mutex_);
-
-            std::cout << "SmartScopesClient.get_search_results():" << std::endl << response_str << std::endl;
-            query_results_.erase(search_id);
-        }
-
-        std::vector<SearchResult> results;
-        std::map<std::string, std::shared_ptr<SearchCategory>> categories;
-
-        std::vector<std::string> jsons = extract_json_stream(response_str);
-
-        for (std::string& json : jsons)
-        {
-            JsonNodeInterface::SPtr root_node;
-            JsonNodeInterface::SPtr child_node;
-
-            {
-                std::lock_guard<std::mutex> lock(json_node_mutex_);
-                json_node_->read_json(json);
-                root_node = json_node_->get_node();
-            }
-
-            if (root_node->has_node("category"))
-            {
-                child_node = root_node->get_node("category");
-                auto category = std::make_shared<SearchCategory>();
-
-                std::vector<std::string> members = child_node->member_names();
-                for (auto& member : members)
-                {
-                    if (member == "icon")
-                    {
-                        category->icon = child_node->get_node(member)->as_string();
-                    }
-                    else if (member == "id")
-                    {
-                        category->id = child_node->get_node(member)->as_string();
-                    }
-                    else if (member == "render_template")
-                    {
-                        category->renderer_template = child_node->get_node(member)->as_string();
-                    }
-                    else if (member == "title")
-                    {
-                        category->title = child_node->get_node(member)->as_string();
-                    }
-                }
-
-                categories[category->id] = category;
-            }
-            else if (root_node->has_node("result"))
-            {
-                child_node = root_node->get_node("result");
-                SearchResult result;
-                result.json = child_node->to_json_string();
-
-                std::vector<std::string> members = child_node->member_names();
-                for (auto& member : members)
-                {
-                    if (member == "uri")
-                    {
-                        result.uri = child_node->get_node(member)->as_string();
-                    }
-                    else if (member == "cat_id")
-                    {
-                        std::string category = child_node->get_node(member)->as_string();
-                        result.category = categories.find(category) != categories.end() ? categories[category] : nullptr;
-                    }
-                    else
-                    {
-                        result.other_params[member] = child_node->get_node(member);
-                    }
-                }
-
-                results.push_back(result);
-            }
-        }
-
-        std::cout << "SmartScopesClient.get_search_results(): Retrieved search results for query " << search_id << std::endl;
-        return results;
+        query_result->get(); // may throw on error
     }
     catch (std::exception const& e)
     {
-        std::cerr << "SmartScopesClient.get_search_results(): Failed to retrieve search results for query " << search_id << std::endl;
+        std::cerr << "SmartScopesClient.get_search_results(): Failed to retrieve search results for query " << search_id << ": " << e.what() << std::endl;
         throw;
     }
+
+    std::lock_guard<std::mutex> lock(query_results_mutex_);
+    query_results_.erase(search_id);
 }
 
-std::pair<PreviewHandle::Columns, PreviewHandle::Widgets> SmartScopesClient::get_preview_results(uint preview_id)
+std::shared_ptr<DepartmentInfo> SmartScopesClient::parse_departments(JsonNodeInterface::SPtr node)
+{
+    static std::array<std::string, 2> const mandatory = { { "label", "canned_query" } };
+    for (auto const& field : mandatory)
+    {
+        if (!node->has_node(field))
+        {
+            std::stringstream err;
+            err << "SmartScopesClient::parse_departments(): missing mandatory department attribute '" << field << "'";
+            throw LogicException(err.str());
+        }
+    }
+
+    auto dep = std::make_shared<DepartmentInfo>();
+    dep->label = node->get_node("label")->as_string();
+    dep->canned_query = node->get_node("canned_query")->as_string();
+
+    if (node->has_node("alternate_label"))
+    {
+        dep->alternate_label = node->get_node("alternate_label")->as_string();
+    }
+    if (node->has_node("has_subdepartments"))
+    {
+        dep->has_subdepartments = node->get_node("has_subdepartments")->as_bool();
+    }
+
+    if (node->has_node("subdepartments"))
+    {
+        auto const subdeps = node->get_node("subdepartments");
+        for (int i = 0; i < subdeps->size(); ++i)
+        {
+            auto child = subdeps->get_node(i);
+            try
+            {
+                dep->subdepartments.push_back(parse_departments(child));
+            }
+            catch (std::exception const& e)
+            {
+                // error in one subdepartment is not critical - just ignore it
+                std::cerr << "SmartScopesClient::parse_departments(): Error parsing subdepartment of department '" << dep->label << "': " << e.what() << std::endl;
+            }
+        }
+        if(subdeps->size() > 0 && dep->subdepartments.size() == 0)
+        {
+            std::stringstream err;
+            err << "SmartScopesClient::parse_departments(): Failed to parse subdepartments of department '" << dep->label;
+            throw LogicException(err.str());
+        }
+    }
+    return dep;
+}
+
+Filters SmartScopesClient::parse_filters(JsonNodeInterface::SPtr node)
+{
+    if (node->type() != JsonNodeInterface::NodeType::Array)
+    {
+        throw LogicException("SmartScopesClient::parse_filters(): 'filters' attribute must be an array");
+    }
+
+    return FilterBaseImpl::deserialize_filters(node->to_variant().get_array());
+}
+
+FilterState SmartScopesClient::parse_filter_state(JsonNodeInterface::SPtr node)
+{
+    if (node->type() != JsonNodeInterface::NodeType::Object)
+    {
+        throw LogicException("SmartScopesClient::parse_filter_state(): 'filter_state' attribute must be an object");
+    }
+
+    return FilterStateImpl::deserialize(node->to_variant().get_dict());
+}
+
+void SmartScopesClient::wait_for_preview(uint preview_id)
 {
     try
     {
         HttpResponseHandle::SPtr query_result;
-        std::string response_str;
-
         {
             std::lock_guard<std::mutex> lock(query_results_mutex_);
 
             auto it = query_results_.find(preview_id);
             if (it == query_results_.end())
             {
-                throw unity::LogicException("No preivew for query " + std::to_string(preview_id) + " is active");
+                throw unity::LogicException("No preview for query " + std::to_string(preview_id) + " is active");
             }
 
             query_result = it->second;
         }
 
         query_result->wait();
-        response_str = query_result->get();
-
-        {
-            std::lock_guard<std::mutex> lock(query_results_mutex_);
-
-            std::cout << "SmartScopesClient.get_preview_results():" << std::endl << response_str << std::endl;
-            query_results_.erase(preview_id);
-        }
-
-        PreviewHandle::Columns columns;
-        PreviewHandle::Widgets widgets;
-
-        std::vector<std::string> jsons = extract_json_stream(response_str);
-
-        for (std::string& json : jsons)
-        {
-            JsonNodeInterface::SPtr root_node;
-            JsonNodeInterface::SPtr child_node;
-
-            {
-                std::lock_guard<std::mutex> lock(json_node_mutex_);
-                json_node_->read_json(json);
-                root_node = json_node_->get_node();
-            }
-
-            if (root_node->has_node("columns"))
-            {
-                child_node = root_node->get_node("columns");
-
-                // for each column
-                for (int column_i = 0; column_i < child_node->size(); ++column_i)
-                {
-                    auto column_node = child_node->get_node(column_i);
-
-                    // for each widget layout within the column
-                    std::vector<std::vector<std::string>> widget_layouts;
-                    for (int widget_lo_i = 0; widget_lo_i < column_node->size(); ++widget_lo_i)
-                    {
-                        auto widget_lo_node = column_node->get_node(widget_lo_i);
-
-                        // for each widget within the widget layout
-                        std::vector<std::string> widget_ids;
-                        for (int widget_i = 0; widget_i < widget_lo_node->size(); ++widget_i)
-                        {
-                            auto widget_node = widget_lo_node->get_node(widget_i);
-                            widget_ids.push_back(widget_node->as_string());
-                        }
-
-                        widget_layouts.push_back(widget_ids);
-                    }
-
-                    columns.push_back(widget_layouts);
-                }
-            }
-            else if (root_node->has_node("widget"))
-            {
-                child_node = root_node->get_node("widget");
-                widgets.push_back(child_node->to_json_string());
-            }
-        }
-
-        std::cout << "SmartScopesClient.get_preview_results(): Retrieved preview results for query " << preview_id << std::endl;
-        return std::make_pair(columns, widgets);
+        query_result->get(); // may throw on error
     }
     catch (std::exception const& e)
     {
         std::cerr << "SmartScopesClient.get_preview_results(): Failed to retrieve preview results for query " << preview_id << std::endl;
         throw;
     }
+
+    std::lock_guard<std::mutex> lock(query_results_mutex_);
+    query_results_.erase(preview_id);
 }
 
 std::vector<std::string> SmartScopesClient::extract_json_stream(std::string const& json_stream)
@@ -649,3 +800,56 @@ std::string SmartScopesClient::read_cache()
 
     return cached_scopes_;
 }
+
+std::string SmartScopesClient::stringify_settings(VariantMap const& settings)
+{
+    std::ostringstream result_str;
+    result_str << "{";
+
+    // Loop through the settings, appending each id:value string to result_str
+    bool first_setting = true;
+    for (auto const& setting : settings)
+    {
+        std::ostringstream setting_str;
+        setting_str << (first_setting ? "\"" : ",\"") << setting.first << "\":";
+
+        // Determine what the Variant type is and construct the id:value string accordingly
+        bool setting_valid = false;
+        switch(setting.second.which())
+        {
+        case Variant::Int:
+            setting_str << setting.second.get_int();
+            setting_valid = true;
+            break;
+        case Variant::Bool:
+            setting_str << (setting.second.get_bool() ? "true" : "false");
+            setting_valid = true;
+            break;
+        case Variant::String:
+            setting_str << "\"" << setting.second.get_string() << "\"";
+            setting_valid = true;
+            break;
+        case Variant::Double:
+            setting_str << setting.second.get_double();
+            setting_valid = true;
+            break;
+        default:
+            std::cerr << "SmartScopesClient.stringify_settings(): Ignoring unsupported Variant type for settings value: \"" << setting.first << "\"" << std::endl;
+        }
+
+        // If we have constructed a valid id:value string, append it to result_str
+        if (setting_valid)
+        {
+            result_str << setting_str.str();
+            first_setting = false;
+        }
+    }
+    result_str << "}";
+
+    // If no valid settings were found, return an empty string
+    if (first_setting)
+    {
+        return std::string();
+    }
+    return result_str.str();
+}
