@@ -18,6 +18,7 @@
 
 #include <unity/scopes/internal/zmq_middleware/ZmqMiddleware.h>
 
+#include <unity/scopes/internal/DfltConfig.h>
 #include <unity/scopes/internal/RuntimeImpl.h>
 #include <unity/scopes/internal/RegistryImpl.h>
 #include <unity/scopes/internal/ScopeImpl.h>
@@ -29,17 +30,21 @@
 #include <unity/scopes/internal/zmq_middleware/ReplyI.h>
 #include <unity/scopes/internal/zmq_middleware/ScopeI.h>
 #include <unity/scopes/internal/zmq_middleware/StateReceiverI.h>
+#include <unity/scopes/internal/zmq_middleware/ZmqConfig.h>
+#include <unity/scopes/internal/zmq_middleware/ZmqPublisher.h>
 #include <unity/scopes/internal/zmq_middleware/ZmqQuery.h>
 #include <unity/scopes/internal/zmq_middleware/ZmqQueryCtrl.h>
 #include <unity/scopes/internal/zmq_middleware/ZmqRegistry.h>
 #include <unity/scopes/internal/zmq_middleware/ZmqReply.h>
 #include <unity/scopes/internal/zmq_middleware/ZmqScope.h>
 #include <unity/scopes/internal/zmq_middleware/ZmqStateReceiver.h>
+#include <unity/scopes/internal/zmq_middleware/ZmqSubscriber.h>
 #include <unity/scopes/internal/zmq_middleware/RethrowException.h>
 #include <unity/scopes/ScopeExceptions.h>
 #include <unity/UnityExceptions.h>
 
 #include <iostream>  // TODO: remove this once logging is added
+#include <sys/stat.h>
 
 using namespace std;
 
@@ -58,27 +63,83 @@ namespace zmq_middleware
 namespace
 {
 
-char const* query_suffix = "-q";  // Appended to server_name_ to create query adapter name
-char const* ctrl_suffix = "-c";   // Appended to server_name_ to create control adapter name
-char const* reply_suffix = "-r";  // Appended to server_name_ to create reply adapter name
-char const* state_suffix = "-s";  // Appended to server_name_ to create state adapter name
+char const* query_suffix = "-q";     // Appended to server_name_ to create query adapter name
+char const* ctrl_suffix = "-c";      // Appended to server_name_ to create control adapter name
+char const* reply_suffix = "-r";     // Appended to server_name_ to create reply adapter name
+char const* state_suffix = "-s";     // Appended to server_name_ to create state adapter name
+char const* registry_suffix = "-R";  // Appended to server_name_ to create registry (or SS registry) adapter name
+char const* publisher_suffix = "-p"; // Appended to publisher_id to create a publisher endpoint
+
+char const* query_category = "Query";       // query adapter category name
+char const* ctrl_category = "QueryCtrl";    // control adapter category name
+char const* reply_category = "Reply";       // reply adapter category name
+char const* state_category = "State";       // state adapter category name
+char const* scope_category = "Scope";       // scope adapter category name
+char const* registry_category = "Registry"; // registry adapter category name
+
+// Create a directory with the given mode if it doesn't exist yet.
+
+void create_dir(string const& dir, mode_t mode)
+{
+    if (mkdir(dir.c_str(), mode) == -1 && errno != EEXIST)
+    {
+        throw FileException("cannot create endpoint directory " + dir, errno);
+    }
+}
 
 } // namespace
 
-ZmqMiddleware::ZmqMiddleware(string const& server_name, string const& configfile, RuntimeImpl* runtime)
-try :
+ZmqMiddleware::ZmqMiddleware(string const& server_name, RuntimeImpl* runtime, string const& configfile) :
     MiddlewareBase(runtime),
     server_name_(server_name),
-    state_(Stopped),
-    config_(configfile),
-    twoway_timeout_(300),  // TODO: get timeout from config
-    locate_timeout_(1500)  // TODO: get timeout from config
+    state_(Created),
+    shutdown_flag(false)
 {
     assert(!server_name.empty());
-}
-catch (zmqpp::exception const& e)
-{
-    rethrow_zmq_ex(e);
+
+    try
+    {
+        ZmqConfig config(configfile);
+
+        twoway_timeout_ = config.twoway_timeout();
+        locate_timeout_ = config.locate_timeout();
+        registry_timeout_ = config.registry_timeout();
+        public_endpoint_dir_ = config.endpoint_dir();
+        private_endpoint_dir_ = public_endpoint_dir_ + "/priv";
+        registry_endpoint_dir_ = public_endpoint_dir_;
+        ss_registry_endpoint_dir_ = public_endpoint_dir_;
+        registry_identity_ = DFLT_REGISTRY_ID;
+        ss_registry_identity_ = DFLT_SS_REGISTRY_ID;
+
+        if (runtime) // runtime can be nullptr for some of the tests. It is never null otherwise.
+        {
+            // The registry and smartscopesproxy can override the normal endpoint dir. That's needed
+            // for testing, so the shell can run a test registry somewhere other than the normal
+            // registry, but still use the normal smartscopesproxy.
+            if (!config.registry_endpoint_dir().empty())
+            {
+                registry_endpoint_dir_ = config.registry_endpoint_dir();
+            }
+            if (!config.ss_registry_endpoint_dir().empty())
+            {
+                ss_registry_endpoint_dir_ = config.ss_registry_endpoint_dir();
+            }
+            registry_identity_ = runtime->registry_identity();
+            ss_registry_identity_ = runtime->ss_registry_identity();
+        }
+
+        // Create the endpoint dirs if they don't exist.
+        // We set the sticky bit because, without this, things in
+        // $XDG_RUNTIME_DIR may be deleted if not accessed for more than six hours.
+        create_dir(public_endpoint_dir_, 0755 | S_ISVTX);
+        create_dir(private_endpoint_dir_, 0700 | S_ISVTX);
+        create_dir(registry_endpoint_dir_, 0755 | S_ISVTX);
+        create_dir(ss_registry_endpoint_dir_, 0755 | S_ISVTX);
+    }
+    catch (...)
+    {
+        throw MiddlewareException("cannot initialize zmq middleware for scope " + server_name);
+    }
 }
 
 ZmqMiddleware::~ZmqMiddleware()
@@ -87,13 +148,35 @@ ZmqMiddleware::~ZmqMiddleware()
     {
         stop();
         wait_for_shutdown();
+
+        // TODO:
+        // We terminate explicitly here instead of relying
+        // on the context_ destructor so we can measure how long it
+        // takes. There is an intermittent problem with
+        // zmq taking several seconds to terminate the context.
+        // Until we figure out what's going on here, we measure
+        // how long it takes and print a warning if it takes
+        // longer than 100 ms.
+        auto start_time = chrono::system_clock::now();
+        context_.terminate();
+        auto end_time = chrono::system_clock::now();
+        auto millisecs = chrono::duration_cast<chrono::milliseconds>(end_time - start_time).count();
+        if (millisecs > 100)
+        {
+            ostringstream s;
+            s << "warning: ~ZmqMiddleware(): context_.terminate() took " << millisecs
+              << " ms to complete for " << server_name_ << endl;
+            cerr << s.str();
+        }
     }
     catch (std::exception const& e)
     {
+        cerr << "~ZmqMiddleware(): " << e.what() << endl;
         // TODO: log exception
     }
     catch (...)
     {
+        cerr << "~ZmqMiddleware(): unknown exception" << endl;
         // TODO: log exception
     }
 }
@@ -105,18 +188,43 @@ void ZmqMiddleware::start()
     switch (state_)
     {
         case Started:
-        case Starting:
         {
-            return; // Already started, or about to start, no-op
+            return; // Already started, no-op
         }
+        case Stopping:
         case Stopped:
         {
-            // TODO: get directory from config
-            // TODO: get pool size from config
+            throw MiddlewareException("Cannot re-start stopped middleware");
+        }
+        case Created:
+        {
             {
                 lock_guard<mutex> lock(data_mutex_);
-                invokers_.reset(new ThreadPool(1));
+                try
+                {
+                    oneway_invoker_.reset(new ThreadPool(1));  // Oneway pool must have a single thread
+                    // N.B. We absolutely MUST have AT LEAST 5 two-way invoke threads:
+                    // * 3 threads are required to execute a standard scope invocation as both
+                    //   rebinding and debug_mode requests could be invoked within a single two-way
+                    //   invocation.
+                    // * We then need an extra thread available per layer of hierarchy under an
+                    //   aggregating scope as an aggregator may invoke a nested scope while running in
+                    //   a thread of its own.
+                    // * 5 threads therefore, at least allows for an aggregating scope to invoke nested
+                    //   aggregators.
+                    // (NOTE: To be safe, we should keep some headroom above this 5 thread minimum)
+                    twoway_invokers_.reset(new ThreadPool(8));  // TODO: get pool size from config
+                }
+                catch (std::exception const& e)
+                {
+                    throw MiddlewareException(string("Cannot create outgoing invocation pools: ") + e.what());
+                }
+                catch (...)
+                {
+                    throw MiddlewareException("Cannot create outgoing invocation pools: unknown exception");
+                }
             }
+            shutdown_flag = false;
             state_ = Started;
             state_changed_.notify_all();
             break;
@@ -134,36 +242,31 @@ void ZmqMiddleware::stop()
     switch (state_)
     {
         case Stopped:
+        case Stopping:
         {
-            break;  // Already stopped, or about to stop, no-op
-        }
-        case Starting:
-        {
-            // Wait until start in progress has completed before stopping
-            // Coverage excluded here because the window for which we are in this state is too
-            // small to hit with a test.
-            state_changed_.wait(lock, [this] { return state_ == Started; }); // LCOV_EXCL_LINE
-            // FALLTHROUGH
+            break;  // Already stopped, about to stop, or never started: no-op
         }
+        case Created:
         case Started:
         {
+            lock_guard<mutex> lock(data_mutex_);
+
+            // No more outgoing invocations
+            assert((oneway_invoker_ && twoway_invokers_) || (!oneway_invoker_ && !twoway_invokers_));
+            if (oneway_invoker_)
             {
-                lock_guard<mutex> lock(data_mutex_);
-                // No more outgoing invocations
-                invokers_.reset();
+                twoway_invokers_->destroy();            // Destroy immediately, because invocations can take time.
+                oneway_invoker_->destroy_once_empty();  // Wait for queued oneways to go out first.
             }
-            auto adapter_map = move(am_);
-            for (auto& pair : adapter_map)
+
+            // Initiate shutdown of all adapters
+            for (auto& pair : am_)
             {
                 pair.second->shutdown();
             }
-            state_ = Stopped;
+
+            state_ = Stopping;
             state_changed_.notify_all();
-            lock.unlock();
-            for (auto& pair : adapter_map)
-            {
-                pair.second->wait_for_shutdown();
-            }
             break;
         }
         default:
@@ -175,8 +278,42 @@ void ZmqMiddleware::stop()
 
 void ZmqMiddleware::wait_for_shutdown()
 {
-    unique_lock<mutex> lock(state_mutex_);
-    state_changed_.wait(lock, [this] { return state_ == Stopped; }); // LCOV_EXCL_LINE
+    {
+        unique_lock<mutex> state_lock(state_mutex_);
+        state_changed_.wait(state_lock, [this] {
+            return state_ == Stopping || state_ == Stopped || state_ == Created;
+        });
+        if (state_ == Stopped)
+        {
+            return; // Return immediately if stopped already, or never started in the first place.
+        }
+
+        // Several threads may see state_ == Stopping here. Exactly one of them
+        // waits for the object adapters to shut down; the others wait until
+        // shut down is complete.
+        if (shutdown_flag.exchange(true))
+        {
+            // Another thread has been through here already, wait for it
+            // to finish shutting down the adapters.
+            state_changed_.wait(state_lock, [this] { return state_ == Stopped; });
+            return;
+        }
+    }
+
+    // Exactly one thread gets to this point.
+    AdapterMap adapter_map;
+    {
+        lock_guard<mutex> data_lock(data_mutex_);
+        adapter_map = move(am_);
+    }
+    for (auto&& pair : adapter_map)
+    {
+        pair.second->wait_for_shutdown();
+    }
+
+    unique_lock<mutex> state_lock(state_mutex_);
+    state_ = Stopped;
+    state_changed_.notify_all();
 }
 
 namespace
@@ -273,7 +410,7 @@ ObjectProxy ZmqMiddleware::string_to_proxy(string const& s)
     }
 
     // Now run over the map and check each value
-    string category = "Scope";
+    string category = scope_category;
     RequestMode mode = RequestMode::Twoway;
     int64_t timeout = -1;
     for (auto const& pair : fmap)
@@ -338,90 +475,65 @@ string ZmqMiddleware::proxy_to_string(MWProxy const& proxy)
     return proxy->to_string();
 }
 
-MWRegistryProxy ZmqMiddleware::create_registry_proxy(string const& identity, string const& endpoint)
+MWRegistryProxy ZmqMiddleware::registry_proxy()
 {
-    MWRegistryProxy proxy;
-    try
+    lock_guard<mutex> lock(data_mutex_);
+
+    if (!registry_proxy_)
     {
-        proxy.reset(new ZmqRegistry(this, endpoint, identity, "Registry", twoway_timeout_));
+        string r_id = registry_identity_;  // May be empty, if no registry is configured.
+        if (!r_id.empty())
+        {
+            string r_endp = "ipc://" + registry_endpoint_dir_ + "/" + r_id + registry_suffix;
+            registry_proxy_.reset(new ZmqRegistry(this, r_endp, r_id, registry_category, twoway_timeout_));
+        }
     }
-    catch (zmqpp::exception const& e)
+    return registry_proxy_;
+}
+
+MWRegistryProxy ZmqMiddleware::ss_registry_proxy()
+{
+    lock_guard<mutex> lock(data_mutex_);
+
+    if (!ss_registry_proxy_)
     {
-        rethrow_zmq_ex(e);
+        string ssr_id = ss_registry_identity_;  // May be empty, if no ss registry is configured.
+        if (!ssr_id.empty())
+        {
+            string ssr_endp = "ipc://" + ss_registry_endpoint_dir_ + "/" + ssr_id + registry_suffix;
+            ss_registry_proxy_.reset(new ZmqRegistry(this, ssr_endp, ssr_id, registry_category, twoway_timeout_));
+        }
     }
-    return proxy;
+    return ss_registry_proxy_;
 }
 
 MWScopeProxy ZmqMiddleware::create_scope_proxy(string const& identity)
 {
-    MWScopeProxy proxy;
-    try
-    {
-        string endpoint = "ipc://" + config_.private_dir() + "/" + identity;
-        proxy.reset(new ZmqScope(this, endpoint, identity, "Scope", twoway_timeout_));
-    }
-    catch (zmqpp::exception const& e)
-    {
-        rethrow_zmq_ex(e);
-    }
-    return proxy;
+    string endpoint = "ipc://" + private_endpoint_dir_ + "/" + identity;
+    return make_shared<ZmqScope>(this, endpoint, identity, scope_category, twoway_timeout_);
 }
 
 MWScopeProxy ZmqMiddleware::create_scope_proxy(string const& identity, string const& endpoint)
 {
-    MWScopeProxy proxy;
-    try
-    {
-        proxy.reset(new ZmqScope(this, endpoint, identity, "Scope", twoway_timeout_));
-    }
-    catch (zmqpp::exception const& e)
-    {
-        rethrow_zmq_ex(e);
-    }
-    return proxy;
+    return make_shared<ZmqScope>(this, endpoint, identity, scope_category, twoway_timeout_);
 }
 
 MWQueryProxy ZmqMiddleware::create_query_proxy(string const& identity, string const& endpoint)
 {
-    MWQueryProxy proxy;
-    try
-    {
-        proxy.reset(new ZmqQuery(this, endpoint, identity, "Query"));
-    }
-    catch (zmqpp::exception const& e)
-    {
-        rethrow_zmq_ex(e);
-    }
-    return proxy;
+    return make_shared<ZmqQuery>(this, endpoint, identity, query_category);
 }
 
 MWQueryCtrlProxy ZmqMiddleware::create_query_ctrl_proxy(string const& identity, string const& endpoint)
 {
-    MWQueryCtrlProxy proxy;
-    try
-    {
-        proxy.reset(new ZmqQueryCtrl(this, endpoint, identity, "QueryCtrl"));
-    }
-    catch (zmqpp::exception const& e)
-    {
-        rethrow_zmq_ex(e);
-    }
-    return proxy;
+    return make_shared<ZmqQueryCtrl>(this, endpoint, identity, ctrl_category);
 }
 
 MWStateReceiverProxy ZmqMiddleware::create_state_receiver_proxy(std::string const& identity)
 {
-    MWStateReceiverProxy proxy;
-    try
-    {
-        proxy.reset(new ZmqStateReceiver(this, "ipc://" + config_.private_dir() + "/" +  server_name_ + state_suffix,
-                                         identity, "StateReceiver"));
-    }
-    catch (zmqpp::exception const& e)
-    {
-        rethrow_zmq_ex(e);
-    }
-    return proxy;
+    // Only override endpoint dir for the registry, not the smartscopes registry.
+    auto endp_dir = (server_name_ == registry_identity_ ? registry_endpoint_dir_ : public_endpoint_dir_);
+    string endpoint = "ipc://" + endp_dir + "/" + server_name_ + state_suffix;
+    return make_shared<ZmqStateReceiver>(this, endpoint, identity, state_category);
 }
 
 MWQueryCtrlProxy ZmqMiddleware::add_query_ctrl_object(QueryCtrlObjectBase::SPtr const& ctrl)
@@ -432,11 +544,12 @@ MWQueryCtrlProxy ZmqMiddleware::add_query_ctrl_object(QueryCtrlObjectBase::SPtr
     try
     {
         shared_ptr<QueryCtrlI> qci(make_shared<QueryCtrlI>(ctrl));
-        auto adapter = find_adapter(server_name_ + ctrl_suffix, config_.private_dir());
+        auto adapter = find_adapter(server_name_ + ctrl_suffix, private_endpoint_dir_, ctrl_category);
         function<void()> df;
-        auto proxy = safe_add(df, adapter, "", qci);
+        auto p = safe_add(df, adapter, "", qci);
         ctrl->set_disconnect_function(df);
-        return ZmqQueryCtrlProxy(new ZmqQueryCtrl(this, proxy->endpoint(), proxy->identity(), "QueryCtrl"));
+        proxy = ZmqQueryCtrlProxy(new ZmqQueryCtrl(this, p->endpoint(), p->identity(), ctrl_category));
+        adapter->activate();
     }
     catch (std::exception const& e) // Should never happen unless our implementation is broken
     {
@@ -454,9 +567,10 @@ void ZmqMiddleware::add_dflt_query_ctrl_object(QueryCtrlObjectBase::SPtr const&
     try
     {
         shared_ptr<QueryCtrlI> qci(make_shared<QueryCtrlI>(ctrl));
-        auto adapter = find_adapter(server_name_ + ctrl_suffix, config_.private_dir());
-        auto df = safe_dflt_add(adapter, "QueryCtrl", qci);
+        auto adapter = find_adapter(server_name_ + ctrl_suffix, private_endpoint_dir_, ctrl_category);
+        auto df = safe_dflt_add(adapter, ctrl_category, qci);
         ctrl->set_disconnect_function(df);
+        adapter->activate();
     }
     catch (std::exception const& e) // Should never happen unless our implementation is broken
     {
@@ -474,11 +588,12 @@ MWQueryProxy ZmqMiddleware::add_query_object(QueryObjectBase::SPtr const& query)
     try
     {
         shared_ptr<QueryI> qi(make_shared<QueryI>(query));
-        auto adapter = find_adapter(server_name_ + query_suffix, config_.private_dir());
+        auto adapter = find_adapter(server_name_ + query_suffix, private_endpoint_dir_, query_category);
         function<void()> df;
-        auto proxy = safe_add(df, adapter, "", qi);
+        auto p = safe_add(df, adapter, "", qi);
         query->set_disconnect_function(df);
-        return ZmqQueryProxy(new ZmqQuery(this, proxy->endpoint(), proxy->identity(), "Query"));
+        proxy = ZmqQueryProxy(new ZmqQuery(this, p->endpoint(), p->identity(), query_category));
+        adapter->activate();
     }
     catch (std::exception const& e) // Should never happen unless our implementation is broken
     {
@@ -496,9 +611,10 @@ void ZmqMiddleware::add_dflt_query_object(QueryObjectBase::SPtr const& query)
     try
     {
         shared_ptr<QueryI> qi(make_shared<QueryI>(query));
-        auto adapter = find_adapter(server_name_ + query_suffix, config_.private_dir());
-        auto df = safe_dflt_add(adapter, "Query", qi);
+        auto adapter = find_adapter(server_name_ + query_suffix, private_endpoint_dir_, query_category);
+        auto df = safe_dflt_add(adapter, query_category, qi);
         query->set_disconnect_function(df);
+        adapter->activate();
     }
     catch (std::exception const& e) // Should never happen unless our implementation is broken
     {
@@ -517,11 +633,13 @@ MWRegistryProxy ZmqMiddleware::add_registry_object(string const& identity, Regis
     try
     {
         shared_ptr<RegistryI> ri(make_shared<RegistryI>(registry));
-        auto adapter = find_adapter(server_name_, runtime()->registry_endpointdir());
+        auto endp_dir = (server_name_ == registry_identity_ ? registry_endpoint_dir_ : ss_registry_endpoint_dir_);
+        auto adapter = find_adapter(server_name_ + registry_suffix, endp_dir, registry_category);
         function<void()> df;
-        auto proxy = safe_add(df, adapter, identity, ri);
+        auto p = safe_add(df, adapter, identity, ri);
         registry->set_disconnect_function(df);
-        return ZmqRegistryProxy(new ZmqRegistry(this, proxy->endpoint(), proxy->identity(), "Registry", twoway_timeout_));
+        proxy = ZmqRegistryProxy(new ZmqRegistry(this, p->endpoint(), p->identity(), registry_category, twoway_timeout_));
+        adapter->activate();
     }
     catch (std::exception const& e) // Should never happen unless our implementation is broken
     {
@@ -540,11 +658,12 @@ MWReplyProxy ZmqMiddleware::add_reply_object(ReplyObjectBase::SPtr const& reply)
     try
     {
         shared_ptr<ReplyI> ri(make_shared<ReplyI>(reply));
-        auto adapter = find_adapter(server_name_ + reply_suffix, config_.public_dir());
+        auto adapter = find_adapter(server_name_ + reply_suffix, public_endpoint_dir_, reply_category);
         function<void()> df;
-        auto proxy = safe_add(df, adapter, "", ri);
+        auto p = safe_add(df, adapter, "", ri);
         reply->set_disconnect_function(df);
-        return ZmqReplyProxy(new ZmqReply(this, proxy->endpoint(), proxy->identity(), "Reply"));
+        proxy = ZmqReplyProxy(new ZmqReply(this, p->endpoint(), p->identity(), reply_category));
+        adapter->activate();
     }
     catch (std::exception const& e) // Should never happen unless our implementation is broken
     {
@@ -555,7 +674,7 @@ MWReplyProxy ZmqMiddleware::add_reply_object(ReplyObjectBase::SPtr const& reply)
     return proxy;
 }
 
-MWScopeProxy ZmqMiddleware::add_scope_object(string const& identity, ScopeObjectBase::SPtr const& scope)
+MWScopeProxy ZmqMiddleware::add_scope_object(string const& identity, ScopeObjectBase::SPtr const& scope, int64_t idle_timeout)
 {
     assert(!identity.empty());
     assert(scope);
@@ -564,11 +683,12 @@ MWScopeProxy ZmqMiddleware::add_scope_object(string const& identity, ScopeObject
     try
     {
         shared_ptr<ScopeI> si(make_shared<ScopeI>(scope));
-        auto adapter = find_adapter(server_name_, config_.private_dir());
+        auto adapter = find_adapter(server_name_, private_endpoint_dir_, scope_category, idle_timeout);
         function<void()> df;
-        auto proxy = safe_add(df, adapter, identity, si);
+        auto p = safe_add(df, adapter, identity, si);
         scope->set_disconnect_function(df);
-        return ZmqScopeProxy(new ZmqScope(this, proxy->endpoint(), proxy->identity(), "Scope", twoway_timeout_));
+        proxy = ZmqScopeProxy(new ZmqScope(this, p->endpoint(), p->identity(), scope_category, twoway_timeout_));
+        adapter->activate();
     }
     catch (std::exception const& e) // Should never happen unless our implementation is broken
     {
@@ -586,9 +706,10 @@ void ZmqMiddleware::add_dflt_scope_object(ScopeObjectBase::SPtr const& scope)
     try
     {
         shared_ptr<ScopeI> si(make_shared<ScopeI>(scope));
-        auto adapter = find_adapter(server_name_, config_.private_dir());
-        auto df = safe_dflt_add(adapter, "Scope", si);
+        auto adapter = find_adapter(server_name_, private_endpoint_dir_, scope_category);
+        auto df = safe_dflt_add(adapter, scope_category, si);
         scope->set_disconnect_function(df);
+        adapter->activate();
     }
     catch (std::exception const& e) // Should never happen unless our implementation is broken
     {
@@ -607,11 +728,14 @@ MWStateReceiverProxy ZmqMiddleware::add_state_receiver_object(std::string const&
     try
     {
         shared_ptr<StateReceiverI> sri(make_shared<StateReceiverI>(state_receiver));
-        auto adapter = find_adapter(server_name_ + state_suffix, config_.private_dir());
+        // Only override endpoint dir for the registry, not the smartscopes registry.
+        auto endp_dir = (server_name_ == registry_identity_ ? registry_endpoint_dir_ : public_endpoint_dir_);
+        auto adapter = find_adapter(server_name_ + state_suffix, endp_dir, state_category);
         function<void()> df;
-        auto proxy = safe_add(df, adapter, identity, sri);
+        auto p = safe_add(df, adapter, identity, sri);
         state_receiver->set_disconnect_function(df);
-        return ZmqStateReceiverProxy(new ZmqStateReceiver(this, proxy->endpoint(), proxy->identity(), "StateReceiver"));
+        proxy = ZmqStateReceiverProxy(new ZmqStateReceiver(this, p->endpoint(), p->identity(), state_category));
+        adapter->activate();
     }
     catch (std::exception const& e) // Should never happen unless our implementation is broken
     {
@@ -622,9 +746,23 @@ MWStateReceiverProxy ZmqMiddleware::add_state_receiver_object(std::string const&
     return proxy;
 }
 
+MWPublisher::UPtr ZmqMiddleware::create_publisher(std::string const& publisher_id)
+{
+    // Only override endpoint dir for the registry, not the smartscopes registry.
+    auto endp_dir = (server_name_ == registry_identity_ ? registry_endpoint_dir_ : public_endpoint_dir_);
+    return MWPublisher::UPtr(new ZmqPublisher(&context_, publisher_id + publisher_suffix, endp_dir));
+}
+
+MWSubscriber::UPtr ZmqMiddleware::create_subscriber(std::string const& publisher_id, std::string const& topic)
+{
+    // Only override endpoint dir for the registry, not the smartscopes registry.
+    auto endp_dir = (server_name_ == registry_identity_ ? registry_endpoint_dir_ : public_endpoint_dir_);
+    return MWSubscriber::UPtr(new ZmqSubscriber(&context_, publisher_id + publisher_suffix, endp_dir, topic));
+}
+
 std::string ZmqMiddleware::get_scope_endpoint()
 {
-    return "ipc://" + config_.private_dir() + "/" +  server_name_;
+    return "ipc://" + private_endpoint_dir_ + "/" +  server_name_;
 }
 
 std::string ZmqMiddleware::get_query_endpoint()
@@ -634,7 +772,7 @@ std::string ZmqMiddleware::get_query_endpoint()
 
 std::string ZmqMiddleware::get_query_ctrl_endpoint()
 {
-    return "ipc://" + config_.private_dir() + "/" +  server_name_ + ctrl_suffix;
+    return "ipc://" + private_endpoint_dir_ + "/" +  server_name_ + ctrl_suffix;
 }
 
 zmqpp::context* ZmqMiddleware::context() const noexcept
@@ -642,20 +780,28 @@ zmqpp::context* ZmqMiddleware::context() const noexcept
     return const_cast<zmqpp::context*>(&context_);
 }
 
-ThreadPool* ZmqMiddleware::invoke_pool()
+ThreadPool* ZmqMiddleware::oneway_pool()
 {
     lock(state_mutex_, data_mutex_);
-    unique_lock<mutex> state_lock(state_mutex_, std::adopt_lock);
+    lock_guard<mutex> state_lock(state_mutex_, std::adopt_lock);
     lock_guard<mutex> invokers_lock(data_mutex_, std::adopt_lock);
-    if (state_ == Starting)
-    {
-        state_changed_.wait(state_lock, [this] { return state_ != Starting; }); // LCOV_EXCL_LINE
-    }
-    if (state_ == Stopped)
+    if (state_ != Started)
     {
         throw MiddlewareException("Cannot invoke operations while middleware is stopped");
     }
-    return invokers_.get();
+    return oneway_invoker_.get();
+}
+
+ThreadPool* ZmqMiddleware::twoway_pool()
+{
+    lock(state_mutex_, data_mutex_);
+    lock_guard<mutex> state_lock(state_mutex_, std::adopt_lock);
+    lock_guard<mutex> invokers_lock(data_mutex_, std::adopt_lock);
+    if (state_ != Started)
+    {
+        throw MiddlewareException("Cannot invoke operations while middleware is stopped");
+    }
+    return twoway_invokers_.get();
 }
 
 int64_t ZmqMiddleware::locate_timeout() const noexcept
@@ -663,6 +809,11 @@ int64_t ZmqMiddleware::locate_timeout() const noexcept
     return locate_timeout_;
 }
 
+int64_t ZmqMiddleware::registry_timeout() const noexcept
+{
+    return registry_timeout_;
+}
+
 ObjectProxy ZmqMiddleware::make_typed_proxy(string const& endpoint,
                                             string const& identity,
                                             string const& category,
@@ -675,12 +826,12 @@ ObjectProxy ZmqMiddleware::make_typed_proxy(string const& endpoint,
     {
         throw MiddlewareException("make_typed_proxy(): cannot create oneway proxies");
     }
-    if (category == "Scope")
+    if (category == scope_category)
     {
         auto p = make_shared<ZmqScope>(this, endpoint, identity, category, timeout);
         return ScopeImpl::create(p, runtime(), identity);
     }
-    else if (category == "Registry")
+    else if (category == registry_category)
     {
         auto p = make_shared<ZmqRegistry>(this, endpoint, identity, category, timeout);
         return make_shared<RegistryImpl>(p, runtime());
@@ -691,25 +842,17 @@ ObjectProxy ZmqMiddleware::make_typed_proxy(string const& endpoint,
     }
 }
 
-namespace
+shared_ptr<ObjectAdapter> ZmqMiddleware::find_adapter(string const& name, string const& endpoint_dir,
+                                                      string const& category, int64_t idle_timeout)
 {
+    lock(state_mutex_, data_mutex_);
+    lock_guard<mutex> state_lock(state_mutex_, std::adopt_lock);
+    lock_guard<mutex> map_lock(data_mutex_, std::adopt_lock);
 
-bool has_suffix(string const& s, string const& suffix)
-{
-    auto s_len = s.length();
-    auto suffix_len = suffix.length();
-    if (s_len >= suffix_len)
+    if (state_ == Stopping || state_ == Stopped)
     {
-        return s.compare(s_len - suffix_len, suffix_len, suffix) == 0;
+        throw MiddlewareException("Cannot invoke operations while middleware is stopped");  // TODO: report mw name
     }
-    return false;
-}
-
-} // namespace
-
-shared_ptr<ObjectAdapter> ZmqMiddleware::find_adapter(string const& name, string const& endpoint_dir)
-{
-    lock_guard<mutex> lock(data_mutex_);
 
     auto it = am_.find(name);
     if (it != am_.end())
@@ -720,33 +863,49 @@ shared_ptr<ObjectAdapter> ZmqMiddleware::find_adapter(string const& name, string
     // We don't have the requested adapter yet, so we create it on the fly.
     int pool_size;
     RequestMode mode;
-    if (has_suffix(name, query_suffix))
+    if (category == query_category)
     {
         // The query adapter is single or multi-threaded and supports oneway operations only.
         // TODO: get pool size from config
         pool_size = 1;
         mode = RequestMode::Oneway;
     }
-    else if (has_suffix(name, ctrl_suffix))
+    else if (category == ctrl_category)
     {
         // The ctrl adapter is single-threaded and supports oneway operations only.
         pool_size = 1;
         mode = RequestMode::Oneway;
     }
-    else if (has_suffix(name, reply_suffix))
+    else if (category == reply_category)
     {
         // The reply adapter is single- or multi-threaded and supports oneway operations only.
         // TODO: get pool size from config
         pool_size = 1;
         mode = RequestMode::Oneway;
     }
-    else if (has_suffix(name, state_suffix))
+    else if (category == state_category)
     {
         // The state adapter is single- or multi-threaded and supports oneway operations only.
         // TODO: get pool size from config
         pool_size = 1;
         mode = RequestMode::Oneway;
     }
+    else if (category == scope_category)
+    {
+        // The scope adapter is single- or multi-threaded and supports twoway operations only.
+        // TODO: get pool size from config
+        pool_size = 1;
+        mode = RequestMode::Twoway;
+    }
+    else if (category == registry_category)
+    {
+        // The registry adapter is multi-threaded and supports twoway operations only.
+        // TODO: get pool size from config
+        // NB: On rebind, locate() is called on this adapter. A scope may then call registry methods during
+        // its start() method, hence we must ensure this adapter has enough threads available to handle this.
+        pool_size = 11;
+        mode = RequestMode::Twoway;
+    }
     else
     {
         // The normal adapter is single- or multi-threaded and supports twoway operations only.
@@ -757,7 +916,7 @@ shared_ptr<ObjectAdapter> ZmqMiddleware::find_adapter(string const& name, string
 
     // The query adapter is always inproc.
     string endpoint;
-    if (has_suffix(name, query_suffix))
+    if (category == query_category)
     {
         endpoint = "inproc://" + name;
     }
@@ -766,8 +925,7 @@ shared_ptr<ObjectAdapter> ZmqMiddleware::find_adapter(string const& name, string
         endpoint = "ipc://" + endpoint_dir + "/" + name;
     }
 
-    shared_ptr<ObjectAdapter> a(new ObjectAdapter(*this, name, endpoint, mode, pool_size));
-    a->activate();
+    auto a = make_shared<ObjectAdapter>(*this, name, endpoint, mode, pool_size, idle_timeout);
     am_[name] = a;
     return a;
 }
