@@ -16,20 +16,18 @@
  * Authored by: James Henstridge <james.henstridge@canonical.com>
  */
 
+#ifndef TEST_TESTSCOPE_H
+#define TEST_TESTSCOPE_H
+
 #include <unity/scopes/ScopeBase.h>
 
-using namespace std;
-using namespace unity::scopes;
-
-class TestScope : public ScopeBase
+class TestScope : public unity::scopes::ScopeBase
 {
 public:
-    virtual int start(string const&, RegistryProxy const &) override;
-
-    virtual void stop() override;
-
-    virtual void run() override;
-
-    virtual SearchQueryBase::UPtr search(CannedQuery const &, SearchMetadata const &) override;
-    virtual PreviewQueryBase::UPtr preview(Result const&, ActionMetadata const &) override;
+    virtual unity::scopes::SearchQueryBase::UPtr search(unity::scopes::CannedQuery const&,
+                                                        unity::scopes::SearchMetadata const&) override;
+    virtual unity::scopes::PreviewQueryBase::UPtr preview(unity::scopes::Result const&,
+                                                          unity::scopes::ActionMetadata const&) override;
 };
+
+#endif
