@@ -18,6 +18,7 @@
 
 #include <unity/scopes/internal/ScopeMetadataImpl.h>
 
+#include <unity/scopes/internal/RuntimeImpl.h>
 #include <unity/scopes/internal/ScopeImpl.h>
 #include <unity/scopes/internal/zmq_middleware/ZmqMiddleware.h>
 #include <unity/scopes/ScopeExceptions.h>
@@ -25,7 +26,6 @@
 
 #include <boost/regex.hpp>  // Use Boost implementation until http://gcc.gnu.org/bugzilla/show_bug.cgi?id=53631 is fixed.
 #include <gtest/gtest.h>
-#include <scope-api-testconfig.h>
 
 using namespace std;
 using namespace unity;
@@ -33,10 +33,13 @@ using namespace unity::scopes;
 using namespace unity::scopes::internal;
 using namespace unity::scopes::internal::zmq_middleware;
 
+string const runtime_ini = TEST_DIR "/Runtime.ini";
+string const zmq_ini = TEST_DIR "/Zmq.ini";
+
 TEST(ScopeMetadataImpl, basic)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/ScopeMetadataImpl/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    auto rt = RuntimeImpl::create("testscope", runtime_ini);
+    ZmqMiddleware mw("testscope", rt.get(), zmq_ini);
 
     unique_ptr<ScopeMetadataImpl> mi(new ScopeMetadataImpl(&mw));
     mi->set_scope_id("scope_id");
@@ -45,6 +48,7 @@ TEST(ScopeMetadataImpl, basic)
     mi->set_display_name("display_name");
     mi->set_description("description");
     mi->set_author("author");
+    mi->set_results_ttl_type(ScopeMetadata::ResultsTtlType::Medium);
 
     // Keep a copy for tests below
     unique_ptr<ScopeMetadataImpl> mi2(new ScopeMetadataImpl(*mi));
@@ -58,6 +62,7 @@ TEST(ScopeMetadataImpl, basic)
     EXPECT_EQ("description", m.description());
     EXPECT_EQ("author", m.author());
     EXPECT_EQ(0, m.appearance_attributes().size());
+    EXPECT_EQ(ScopeMetadata::ResultsTtlType::Medium, m.results_ttl_type());
 
     // Check that optional fields that are not set throw
     try
@@ -110,9 +115,22 @@ TEST(ScopeMetadataImpl, basic)
         EXPECT_STREQ("unity::scopes::NotFoundException: attribute not set (name = scope_directory)", e.what());
     }
 
+    try
+    {
+        m.settings_definitions();
+        FAIL();
+    }
+    catch (NotFoundException const& e)
+    {
+        EXPECT_STREQ("unity::scopes::NotFoundException: attribute not set (name = settings_definitions)", e.what());
+    }
+
     // when "invisible" is not set, false is returned
     EXPECT_FALSE(m.invisible());
 
+    // when "location_data_needed" is not set, false is returned
+    EXPECT_FALSE(m.location_data_needed());
+
     // Check that the copy has the same values as the original
     EXPECT_EQ("scope_id", mi2->scope_id());
     EXPECT_EQ("identity", mi2->proxy()->identity());
@@ -121,6 +139,7 @@ TEST(ScopeMetadataImpl, basic)
     EXPECT_EQ("description", mi2->description());
     EXPECT_EQ("author", mi2->author());
     EXPECT_EQ(0, mi2->appearance_attributes().size());
+    EXPECT_EQ(ScopeMetadata::ResultsTtlType::Medium, mi2->results_ttl_type());
 
     VariantMap attrs;
     attrs["foo"] = "bar";
@@ -133,6 +152,11 @@ TEST(ScopeMetadataImpl, basic)
     mi2->set_invisible(true);
     mi2->set_appearance_attributes(attrs);
     mi2->set_scope_directory("/foo");
+    mi2->set_results_ttl_type(ScopeMetadata::ResultsTtlType::Large);
+    VariantArray va;
+    va.push_back(Variant("hello"));
+    mi2->set_settings_definitions(va);
+    mi2->set_location_data_needed(true);
 
     // Make another copy, so we get coverage on the entire copy constructor
     unique_ptr<ScopeMetadataImpl> mi3(new ScopeMetadataImpl(*mi2));
@@ -143,6 +167,8 @@ TEST(ScopeMetadataImpl, basic)
     EXPECT_EQ("hot_key", m.hot_key());
     EXPECT_TRUE(m.invisible());
     EXPECT_EQ("bar", m.appearance_attributes()["foo"].get_string());
+    EXPECT_EQ(va, m.settings_definitions());
+    EXPECT_TRUE(m.location_data_needed());
 
     // Make another value
     unique_ptr<ScopeMetadataImpl> ti(new ScopeMetadataImpl(&mw));
@@ -159,6 +185,11 @@ TEST(ScopeMetadataImpl, basic)
     ti->set_invisible(true);
     ti->set_scope_directory("/foo");
     ti->set_appearance_attributes(attrs);
+    ti->set_results_ttl_type(ScopeMetadata::ResultsTtlType::Small);
+    VariantArray tmp_va;
+    tmp_va.push_back(Variant("tmp hello"));
+    ti->set_settings_definitions(tmp_va);
+    ti->set_location_data_needed(true);
 
     // Check impl assignment operator
     ScopeMetadataImpl ci(&mw);
@@ -176,6 +207,9 @@ TEST(ScopeMetadataImpl, basic)
     EXPECT_EQ("/foo", ci.scope_directory());
     EXPECT_EQ("bar", ci.appearance_attributes()["foo"].get_string());
     EXPECT_TRUE(ci.invisible());
+    EXPECT_EQ(ScopeMetadata::ResultsTtlType::Small, ci.results_ttl_type());
+    EXPECT_EQ(tmp_va, ci.settings_definitions());
+    EXPECT_TRUE(ci.location_data_needed());
 
     // Check public assignment operator
     auto tmp = ScopeMetadataImpl::create(move(ti));
@@ -192,7 +226,10 @@ TEST(ScopeMetadataImpl, basic)
     EXPECT_EQ("tmp hot_key", m.hot_key());
     EXPECT_EQ("/foo", m.scope_directory());
     EXPECT_EQ("bar", m.appearance_attributes()["foo"].get_string());
+    EXPECT_EQ(ScopeMetadata::ResultsTtlType::Small, m.results_ttl_type());
     EXPECT_TRUE(m.invisible());
+    EXPECT_EQ(tmp_va, m.settings_definitions());
+    EXPECT_TRUE(m.location_data_needed());
 
     // Self-assignment
     tmp = tmp;
@@ -208,7 +245,10 @@ TEST(ScopeMetadataImpl, basic)
     EXPECT_EQ("tmp hot_key", tmp.hot_key());
     EXPECT_EQ("bar", tmp.appearance_attributes()["foo"].get_string());
     EXPECT_EQ("/foo", tmp.scope_directory());
+    EXPECT_EQ(ScopeMetadata::ResultsTtlType::Small, tmp.results_ttl_type());
     EXPECT_TRUE(tmp.invisible());
+    EXPECT_EQ(tmp_va, tmp.settings_definitions());
+    EXPECT_TRUE(tmp.location_data_needed());
 
     // Copy constructor
     ScopeMetadata tmp2(tmp);
@@ -224,13 +264,16 @@ TEST(ScopeMetadataImpl, basic)
     EXPECT_EQ("tmp hot_key", tmp2.hot_key());
     EXPECT_EQ("/foo", tmp2.scope_directory());
     EXPECT_EQ("bar", tmp2.appearance_attributes()["foo"].get_string());
+    EXPECT_EQ(ScopeMetadata::ResultsTtlType::Small, tmp2.results_ttl_type());
     EXPECT_TRUE(tmp2.invisible());
+    EXPECT_EQ(tmp_va, tmp2.settings_definitions());
+    EXPECT_TRUE(tmp2.location_data_needed());
 }
 
 TEST(ScopeMetadataImpl, serialize)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/ScopeMetadataImpl/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    auto rt = RuntimeImpl::create("testscope", runtime_ini);
+    ZmqMiddleware mw("testscope", rt.get(), zmq_ini);
 
     unique_ptr<ScopeMetadataImpl> mi(new ScopeMetadataImpl(&mw));
     mi->set_scope_id("scope_id");
@@ -245,11 +288,17 @@ TEST(ScopeMetadataImpl, serialize)
     mi->set_hot_key("hot_key");
     mi->set_scope_directory("/foo");
     mi->set_invisible(false);
+    mi->set_results_ttl_type(ScopeMetadata::ResultsTtlType::Large);
+    VariantArray va;
+    va.push_back(Variant("hello"));
+    va.push_back(Variant("world"));
+    mi->set_settings_definitions(va);
+    mi->set_location_data_needed(false);
 
     // Check that serialize() sets the map values correctly
     auto m = ScopeMetadataImpl::create(move(mi));
     auto var = m.serialize();
-    EXPECT_EQ(11u, var.size());
+    EXPECT_EQ(14u, var.size());
     EXPECT_EQ("scope_id", var["scope_id"].get_string());
     EXPECT_EQ("display_name", var["display_name"].get_string());
     EXPECT_EQ("description", var["description"].get_string());
@@ -260,6 +309,11 @@ TEST(ScopeMetadataImpl, serialize)
     EXPECT_EQ("hot_key", var["hot_key"].get_string());
     EXPECT_EQ("/foo", var["scope_dir"].get_string());
     EXPECT_FALSE(var["invisible"].get_bool());
+    EXPECT_EQ(ScopeMetadata::ResultsTtlType::Large,
+            (ScopeMetadata::ResultsTtlType ) var["results_ttl_type"].get_int());
+    EXPECT_FALSE(var["invisible"].get_bool());
+    EXPECT_EQ(va, var["settings_definitions"].get_array());
+    EXPECT_FALSE(var["location_data_needed"].get_bool());
 
     // Make another instance from the VariantMap and check its fields
     ScopeMetadataImpl c(var, &mw);
@@ -273,13 +327,18 @@ TEST(ScopeMetadataImpl, serialize)
     EXPECT_EQ("icon", c.icon());
     EXPECT_EQ("search_hint", c.search_hint());
     EXPECT_EQ("hot_key", c.hot_key());
+    EXPECT_EQ("/foo", c.scope_directory());
     EXPECT_FALSE(c.invisible());
+    EXPECT_EQ(ScopeMetadata::ResultsTtlType::Large, c.results_ttl_type());
+    EXPECT_FALSE(c.invisible());
+    EXPECT_EQ(va, c.settings_definitions());
+    EXPECT_FALSE(c.location_data_needed());
 }
 
 TEST(ScopeMetadataImpl, serialize_exceptions)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/ScopeMetadataImpl/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    auto rt = RuntimeImpl::create("testscope", runtime_ini);
+    ZmqMiddleware mw("testscope", rt.get(), zmq_ini);
 
     ScopeMetadataImpl mi(&mw);
     try
@@ -345,8 +404,8 @@ TEST(ScopeMetadataImpl, serialize_exceptions)
 
 TEST(ScopeMetadataImpl, deserialize_exceptions)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/ScopeMetadataImpl/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    auto rt = RuntimeImpl::create("testscope", runtime_ini);
+    ZmqMiddleware mw("testscope", rt.get(), zmq_ini);
 
     VariantMap m;
     try
@@ -432,6 +491,7 @@ TEST(ScopeMetadataImpl, deserialize_exceptions)
                      "'description' is missing",
                      e.what());
     }
+
     m["description"] = "description";
     try
     {
@@ -445,13 +505,34 @@ TEST(ScopeMetadataImpl, deserialize_exceptions)
                      "'author' is missing",
                      e.what());
     }
+
     m["author"] = "author";
+    m["results_ttl_type"] = -1;
+    try
+    {
+        ScopeMetadataImpl mi(m, &mw);
+        mi.deserialize(m);
+        FAIL();
+    }
+    catch (InvalidArgumentException const&e)
+    {
+        EXPECT_STREQ("unity::InvalidArgumentException: ScopeMetadata::deserialize(): "
+                     "invalid attribute 'results_ttl_type' with value -1",
+                     e.what());
+    }
 
     // Optional attributes
     m["art"] = "art";
     m["icon"] = "icon";
     m["search_hint"] = "search_hint";
     m["hot_key"] = "hot_key";
+    m["scope_dir"] = "scope_dir";
+    m["invisible"] = true;
+    VariantMap appearance;
+    appearance["a1"] = "a1";
+    m["appearance_attributes"] = appearance;
+    m["results_ttl_type"] = 0;
+    m["location_data_needed"] = true;
 
     ScopeMetadataImpl mi(m, &mw);
     mi.deserialize(m);
@@ -465,4 +546,9 @@ TEST(ScopeMetadataImpl, deserialize_exceptions)
     EXPECT_EQ("icon", mi.icon());
     EXPECT_EQ("search_hint", mi.search_hint());
     EXPECT_EQ("hot_key", mi.hot_key());
+    EXPECT_EQ("scope_dir", mi.scope_directory());
+    EXPECT_TRUE(mi.invisible());
+    EXPECT_EQ(appearance, mi.appearance_attributes());
+    EXPECT_EQ(ScopeMetadata::ResultsTtlType::None, mi.results_ttl_type());
+    EXPECT_TRUE(mi.location_data_needed());
 }
