@@ -19,6 +19,7 @@
 #include <unity/scopes/internal/zmq_middleware/ObjectAdapter.h>
 
 #include <scopes/internal/zmq_middleware/capnproto/Message.capnp.h>
+#include <unity/scopes/internal/RuntimeImpl.h>
 #include <unity/scopes/internal/zmq_middleware/ServantBase.h>
 #include <unity/scopes/internal/zmq_middleware/ZmqException.h>
 #include <unity/scopes/internal/zmq_middleware/ZmqMiddleware.h>
@@ -30,7 +31,6 @@
 #include <boost/regex.hpp>  // Use Boost implementation until http://gcc.gnu.org/bugzilla/show_bug.cgi?id=53631 is fixed.
 #include <capnp/serialize.h>
 #include <gtest/gtest.h>
-#include <scope-api-testconfig.h>
 
 using namespace std;
 using namespace unity;
@@ -38,13 +38,16 @@ using namespace unity::scopes;
 using namespace unity::scopes::internal;
 using namespace unity::scopes::internal::zmq_middleware;
 
+string const runtime_ini = TEST_DIR "/Runtime.ini";
+string const zmq_ini = TEST_DIR "/Zmq.ini";
+
 // We use this to sleep in between adapter creation and shutdown. That's
 // necessary because zmq closes sockets asynchronously. Without the wait,
 // because we are binding to the same endpoint each time, we can get
 // an occastional "address in use" exception because the new socket
 // tries to bind while the old one is still in the process of destroying itself.
 
-void wait(int millisec = 20)
+void wait(int millisec = 200)
 {
     this_thread::sleep_for(chrono::milliseconds(millisec));
 }
@@ -53,8 +56,7 @@ void wait(int millisec = 20)
 
 TEST(ObjectAdapter, basic)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/zmq_middleware/ObjectAdapter/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    ZmqMiddleware mw("testscope", nullptr, zmq_ini);
 
     // Instantiate and destroy oneway and twoway adapters with single and multiple threads.
     {
@@ -132,8 +134,7 @@ TEST(ObjectAdapter, basic)
 
 TEST(ObjectAdapter, state_change)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/zmq_middleware/ObjectAdapter/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    ZmqMiddleware mw("testscope", nullptr, zmq_ini);
 
     {
         wait();
@@ -176,7 +177,7 @@ TEST(ObjectAdapter, state_change)
         }
         catch (MiddlewareException const& e)
         {
-            EXPECT_STREQ("unity::scopes::MiddlewareException: Object adapter in Destroyed "
+            EXPECT_STREQ("unity::scopes::MiddlewareException: activate(): Object adapter in Destroyed "
                          "state (adapter: testscope)",
                          e.what());
         }
@@ -194,25 +195,22 @@ TEST(ObjectAdapter, state_change)
         }
         catch (MiddlewareException const& e)
         {
-            EXPECT_STREQ("unity::scopes::MiddlewareException: ObjectAdapter::run_workers(): broker thread "
+            EXPECT_STREQ("unity::scopes::MiddlewareException: ObjectAdapter::run_workers(): stop thread "
                          "failure (adapter: testscope):\n"
-                         "    unity::scopes::MiddlewareException: ObjectAdapter: broker thread failure "
-                         "(adapter: testscope):\n"
+                         "    unity::scopes::MiddlewareException: StopPublisher(): publisher thread "
+                         "failed (endpoint: inproc://testscope-stopper):\n"
                          "        Address already in use",
                          e.what());
         }
+
         try
         {
             b.shutdown();
         }
         catch (MiddlewareException const& e)
         {
-            EXPECT_STREQ("unity::scopes::MiddlewareException: Object adapter in Failed state (adapter: testscope)\n"
-                         "    Exception history:\n"
-                         "        Exception #1:\n"
-                         "            unity::scopes::MiddlewareException: ObjectAdapter: broker thread failure "
-                         "(adapter: testscope):\n"
-                         "                Address already in use",
+            EXPECT_STREQ("unity::scopes::MiddlewareException: shutdown() [state_ == Failed]: "
+                         "Object adapter in Failed state (adapter: testscope)",
                          e.what());
         }
         try
@@ -221,21 +219,16 @@ TEST(ObjectAdapter, state_change)
         }
         catch (MiddlewareException const& e)
         {
-            EXPECT_STREQ("unity::scopes::MiddlewareException: Object adapter in Failed state (adapter: testscope)\n"
-                          "    Exception history:\n"
-                          "        Exception #1:\n"
-                          "            unity::scopes::MiddlewareException: ObjectAdapter: broker thread failure "
-                          "(adapter: testscope):\n"
-                          "                Address already in use",
-                          e.what());
+            EXPECT_STREQ("unity::scopes::MiddlewareException: wait_for_shutdown(): "
+                         "Object adapter in Failed state (adapter: testscope)",
+                         e.what());
         }
     }
 }
 
 TEST(ObjectAdapter, wait_for_shutdown)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/zmq_middleware/ObjectAdapter/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    ZmqMiddleware mw("testscope", nullptr, zmq_ini);
 
     // Start the adapter and call shutdown() from a different thread after a delay, and wait for the
     // shutdown to complete. We check that the shutdown happens after at least the delay that was specified,
@@ -281,8 +274,8 @@ public:
 
 TEST(ObjectAdapter, add_remove_find)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/zmq_middleware/ObjectAdapter/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    auto rt = RuntimeImpl::create("testscope", runtime_ini);
+    ZmqMiddleware mw("testscope", rt.get(), zmq_ini);
 
     wait();
     ObjectAdapter a(mw, "testscope", "ipc://testscope", RequestMode::Twoway, 5);
@@ -343,8 +336,7 @@ TEST(ObjectAdapter, add_remove_find)
 
 TEST(ObjectAdapter, dispatch_oneway_to_twoway)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/zmq_middleware/ObjectAdapter/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    ZmqMiddleware mw("testscope", nullptr, zmq_ini);
 
     wait();
     ObjectAdapter a(mw, "testscope", "ipc://testscope", RequestMode::Twoway, 1);
@@ -379,8 +371,7 @@ TEST(ObjectAdapter, dispatch_oneway_to_twoway)
 
 TEST(ObjectAdapter, dispatch_twoway_to_oneway)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/zmq_middleware/ObjectAdapter/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    ZmqMiddleware mw("testscope", nullptr, zmq_ini);
 
     wait();
     ObjectAdapter a(mw, "testscope", "ipc://testscope", RequestMode::Oneway, 1);
@@ -406,8 +397,7 @@ TEST(ObjectAdapter, dispatch_twoway_to_oneway)
 
 TEST(ObjectAdapter, dispatch_not_exist)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/zmq_middleware/ObjectAdapter/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    ZmqMiddleware mw("testscope", nullptr, zmq_ini);
 
     wait();
     ObjectAdapter a(mw, "testscope", "ipc://testscope", RequestMode::Twoway, 1);
@@ -448,8 +438,7 @@ TEST(ObjectAdapter, dispatch_not_exist)
 
 TEST(ObjectAdapter, bad_header)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/zmq_middleware/ObjectAdapter/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    ZmqMiddleware mw("testscope", nullptr, zmq_ini);
 
     {
         wait();
@@ -509,8 +498,7 @@ TEST(ObjectAdapter, bad_header)
 
 TEST(ObjectAdapter, corrupt_header)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/zmq_middleware/ObjectAdapter/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    ZmqMiddleware mw("testscope", nullptr, zmq_ini);
 
     {
         wait();
@@ -568,8 +556,8 @@ TEST(ObjectAdapter, corrupt_header)
 
 TEST(ObjectAdapter, invoke_ok)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/zmq_middleware/ObjectAdapter/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    auto rt = RuntimeImpl::create("testscope", runtime_ini);
+    ZmqMiddleware mw("testscope", rt.get(), zmq_ini);
 
     wait();
     ObjectAdapter a(mw, "testscope", "ipc://testscope", RequestMode::Twoway, 1);
@@ -620,8 +608,8 @@ public:
 
 TEST(ObjectAdapter, invoke_object_not_exist)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/zmq_middleware/ObjectAdapter/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    auto rt = RuntimeImpl::create("testscope", runtime_ini);
+    ZmqMiddleware mw("testscope", rt.get(), zmq_ini);
 
     wait();
     ObjectAdapter a(mw, "testscope", "ipc://testscope", RequestMode::Twoway, 1);
@@ -665,8 +653,8 @@ TEST(ObjectAdapter, invoke_object_not_exist)
 
 TEST(ObjectAdapter, invoke_operation_not_exist)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/zmq_middleware/ObjectAdapter/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    auto rt = RuntimeImpl::create("testscope", runtime_ini);
+    ZmqMiddleware mw("testscope", rt.get(), zmq_ini);
 
     wait();
     ObjectAdapter a(mw, "testscope", "ipc://testscope", RequestMode::Twoway, 1);
@@ -754,9 +742,10 @@ private:
     int delay_;
 };
 
-void invoke_thread(ZmqMiddleware* mw, RequestMode t)
+void invoke_thread(ZmqMiddleware* mw, RequestMode t, string const& object_id)
 {
     zmqpp::socket s(*mw->context(), t == RequestMode::Twoway ? zmqpp::socket_type::request : zmqpp::socket_type::push);
+    s.set(zmqpp::socket_option::linger, 200);
     s.connect("ipc://testscope");
     ZmqSender sender(s);
     ZmqReceiver receiver(s);
@@ -764,7 +753,7 @@ void invoke_thread(ZmqMiddleware* mw, RequestMode t)
     capnp::MallocMessageBuilder b;
     auto request = b.initRoot<capnproto::Request>();
     request.setMode(t == RequestMode::Twoway ? capnproto::RequestMode::TWOWAY : capnproto::RequestMode::ONEWAY);
-    request.setId("some_id");
+    request.setId(object_id);
     request.setCat("some_cat");
     request.setOpName("count_op");
 
@@ -777,16 +766,12 @@ void invoke_thread(ZmqMiddleware* mw, RequestMode t)
         auto response = reader.getRoot<capnproto::Response>();
         EXPECT_EQ(response.getStatus(), capnproto::ResponseStatus::SUCCESS);
     }
-    else
-    {
-        wait(50);  // Allow some time for oneway requests to actually make it on the wire.
-    }
 }
 
 TEST(ObjectAdapter, twoway_threading)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/zmq_middleware/ObjectAdapter/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    auto rt = RuntimeImpl::create("testscope", runtime_ini);
+    ZmqMiddleware mw("testscope", rt.get(), zmq_ini);
 
     // Single servant to which we send requests concurrently.
     shared_ptr<CountingServant> o(new CountingServant(100));
@@ -803,7 +788,7 @@ TEST(ObjectAdapter, twoway_threading)
         vector<thread> invokers;
         for (auto i = 0; i < num_requests; ++i)
         {
-            invokers.push_back(thread(invoke_thread, &mw, RequestMode::Twoway));
+            invokers.push_back(thread(invoke_thread, &mw, RequestMode::Twoway, "some_id"));
         }
         for (auto& i : invokers)
         {
@@ -819,8 +804,8 @@ TEST(ObjectAdapter, twoway_threading)
 
 TEST(ObjectAdapter, oneway_threading)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/zmq_middleware/ObjectAdapter/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    auto rt = RuntimeImpl::create("testscope", runtime_ini);
+    ZmqMiddleware mw("testscope", rt.get(), zmq_ini);
 
     // Single servant to which we send requests concurrently.
     shared_ptr<CountingServant> o(new CountingServant(100));
@@ -837,7 +822,7 @@ TEST(ObjectAdapter, oneway_threading)
         vector<thread> invokers;
         for (auto i = 0; i < num_requests; ++i)
         {
-            invokers.push_back(thread(invoke_thread, &mw, RequestMode::Oneway));
+            invokers.push_back(thread(invoke_thread, &mw, RequestMode::Oneway, "some_id"));
         }
         for (auto& i : invokers)
         {
@@ -859,6 +844,121 @@ TEST(ObjectAdapter, oneway_threading)
     EXPECT_EQ(num_threads, o->max_concurrent());
 }
 
+// Show that a slow twoway invocation does not delay processing of other twoway invocations if
+// the number of outstanding invocations exceeds the number of worker threads.
+
+TEST(ObjectAdapter, load_balancing_twoway)
+{
+    auto rt = RuntimeImpl::create("testscope", runtime_ini);
+    ZmqMiddleware mw("testscope", rt.get(), zmq_ini);
+
+    // Twoway adapter with 3 threads.
+    ObjectAdapter a(mw, "testscope", "ipc://testscope", RequestMode::Twoway, 3);
+    a.activate();
+
+    // Add servants that take 50 ms (fast) and 1000 ms (slow)
+    shared_ptr<CountingServant> slow_servant(new CountingServant(1000));
+    shared_ptr<CountingServant> fast_servant(new CountingServant(50));
+    a.add("slow", slow_servant);
+    a.add("fast", fast_servant);
+
+    // Send a single request to the slow servant, and 30 requests to the fast servant.
+    // The slow servant ties up a thread for a second, so the other two threads
+    // should be processing the fast invocations during that time, meaning that the
+    // 31 requests should complete in about a second.
+
+    auto start_time = chrono::system_clock::now();
+
+    vector<thread> invokers;
+    invokers.push_back(thread(invoke_thread, &mw, RequestMode::Twoway, "slow"));
+    for (auto i = 0; i < 30; ++i)
+    {
+        invokers.push_back(thread(invoke_thread, &mw, RequestMode::Twoway, "fast"));
+    }
+    for (auto& i : invokers)
+    {
+        i.join();
+    }
+
+    // We set a generous limit of two seconds, even though the entire thing will normally
+    // finish in about 1.1 seconds, in case we are slow on Jenkins.
+    auto end_time = chrono::system_clock::now();
+    EXPECT_LT(chrono::duration_cast<chrono::milliseconds>(end_time - start_time).count(), 2000);
+
+    // We must have had 1 request on the slow servant, and 30 on the fast servant, with the
+    // fast servant getting at least 2 invocations at the same time. Depending on scheduling
+    // order, it's possible for a fast request to be sent before the single slow request,
+    // we may have max concurrency of 3 in the fast servant occasionally.)
+    EXPECT_EQ(1, slow_servant->num_invocations());
+    EXPECT_EQ(30, fast_servant->num_invocations());
+    EXPECT_GE(fast_servant->max_concurrent(), 2);
+}
+
+// Show that a slow oneway invocation does not delay processing of other oneway invocations if
+// the number of outstanding invocations exceeds the number of worker threads.
+
+TEST(ObjectAdapter, load_balancing_oneway)
+{
+    auto rt = RuntimeImpl::create("testscope", runtime_ini);
+    ZmqMiddleware mw("testscope", rt.get(), zmq_ini);
+
+    // Oneway adapter with 3 threads.
+    ObjectAdapter a(mw, "testscope", "ipc://testscope", RequestMode::Oneway, 3);
+    a.activate();
+
+    // Add servants that take 10 ms (fast) and 1000 ms (slow)
+    shared_ptr<CountingServant> slow_servant(new CountingServant(1000));
+    shared_ptr<CountingServant> fast_servant(new CountingServant(10));
+    a.add("slow", slow_servant);
+    a.add("fast", fast_servant);
+
+    // Socket to invoke on adapter
+    zmqpp::socket s(*mw.context(), zmqpp::socket_type::push);
+    s.set(zmqpp::socket_option::linger, 200);
+    s.connect("ipc://testscope");
+    ZmqSender sender(s);
+
+    // Request for invoking slow servant.
+    capnp::MallocMessageBuilder slow_b;
+    auto slow_req = slow_b.initRoot<capnproto::Request>();
+    slow_req.setMode(capnproto::RequestMode::ONEWAY);
+    slow_req.setId("slow");
+    slow_req.setCat("some_cat");
+    slow_req.setOpName("count_op");
+
+    // Request for invoking fast servant.
+    capnp::MallocMessageBuilder fast_b;
+    auto fast_req = fast_b.initRoot<capnproto::Request>();
+    fast_req.setMode(capnproto::RequestMode::ONEWAY);
+    fast_req.setId("fast");
+    fast_req.setCat("some_cat");
+    fast_req.setOpName("count_op");
+
+    // Send a single request to the slow servant, and 140 requests to the fast servant.
+    // The slow servant ties up a thread for a second, so the other two threads
+    // are processing the fast invocations during that time, meaning that the
+    // 141 requests should complete in about a second.
+    auto slow_segments = slow_b.getSegmentsForOutput();
+    sender.send(slow_segments);
+
+    auto fast_segments = fast_b.getSegmentsForOutput();
+    for (int i = 0; i < 140; ++i)
+    {
+        sender.send(fast_segments);
+    }
+
+    // Oneway invocations, so we need to give them a chance to finish.
+    // We set a generous limit of two seconds, even though the entire thing will normally
+    // finish in about 1.1 seconds, in case we are slow on Jenkins.
+    this_thread::sleep_for(chrono::seconds(2));
+
+    // We must have had 1 request on the slow servant, and 140 on the fast servant, with the
+    // fast servant getting 2 invocations concurrently.
+    EXPECT_EQ(1, slow_servant->num_invocations());
+    EXPECT_EQ(140, fast_servant->num_invocations());
+    EXPECT_EQ(2, fast_servant->max_concurrent());
+}
+
 using namespace std::placeholders;
 
 // Servant that updates the servant map in various ways from its destructor, to verify
@@ -890,8 +990,8 @@ public:
 
 TEST(ObjectAdapter, servant_map_destructor)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/zmq_middleware/ObjectAdapter/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    auto rt = RuntimeImpl::create("testscope", runtime_ini);
+    ZmqMiddleware mw("testscope", rt.get(), zmq_ini);
 
     {
         wait();
@@ -907,7 +1007,8 @@ TEST(ObjectAdapter, servant_map_destructor)
             }
             catch (MiddlewareException const& e)
             {
-                EXPECT_STREQ("unity::scopes::MiddlewareException: Object adapter in Destroyed state (adapter: testscope)",
+                EXPECT_STREQ("unity::scopes::MiddlewareException: remove(): "
+                             "Object adapter in Destroyed state (adapter: testscope)",
                              e.what());
             }
         };
@@ -1005,7 +1106,7 @@ TEST(ObjectAdapter, servant_map_destructor)
         }
         catch (MiddlewareException const& e)
         {
-            EXPECT_STREQ("unity::scopes::MiddlewareException: Object adapter in Destroyed "
+            EXPECT_STREQ("unity::scopes::MiddlewareException: add(): Object adapter in Destroyed "
                          "state (adapter: testscope)",
                          e.what());
         }
@@ -1016,7 +1117,7 @@ TEST(ObjectAdapter, servant_map_destructor)
         }
         catch (MiddlewareException const& e)
         {
-            EXPECT_STREQ("unity::scopes::MiddlewareException: Object adapter in Destroyed "
+            EXPECT_STREQ("unity::scopes::MiddlewareException: find(): Object adapter in Destroyed "
                          "state (adapter: testscope)",
                          e.what());
         }
@@ -1061,8 +1162,8 @@ TEST(ObjectAdapter, double_bind)
 
         wait(200);  // Give zmq some time to finish the bind.
 
-        ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/zmq_middleware/ObjectAdapter/Zmq.ini",
-                         (RuntimeImpl*)0x1);
+        ZmqMiddleware mw("testscope", nullptr, zmq_ini);
+
         wait();
         try
         {
@@ -1073,10 +1174,10 @@ TEST(ObjectAdapter, double_bind)
         }
         catch (MiddlewareException const& e)
         {
-            EXPECT_STREQ("unity::scopes::MiddlewareException: ObjectAdapter::run_workers(): broker thread failure "
+            EXPECT_STREQ("unity::scopes::MiddlewareException: ObjectAdapter: pump thread failure "
                          "(adapter: testscope):\n"
-                         "    unity::scopes::MiddlewareException: ObjectAdapter: broker thread failure "
-                         "(adapter: testscope): address in use: ipc://testscope", e.what());
+                         "    unity::scopes::MiddlewareException: safe_bind(): address in use: ipc://testscope",
+                         e.what());
         }
 
         {
@@ -1098,8 +1199,7 @@ TEST(ObjectAdapter, double_bind)
 
         wait(200);  // Give zmq some time to finish the bind.
 
-        ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/zmq_middleware/ObjectAdapter/Zmq.ini",
-                         (RuntimeImpl*)0x1);
+        ZmqMiddleware mw("testscope", nullptr, zmq_ini);
         wait();
         try
         {
@@ -1110,10 +1210,10 @@ TEST(ObjectAdapter, double_bind)
         }
         catch (MiddlewareException const& e)
         {
-            EXPECT_STREQ("unity::scopes::MiddlewareException: ObjectAdapter::run_workers(): broker thread failure "
+            EXPECT_STREQ("unity::scopes::MiddlewareException: ObjectAdapter: pump thread failure "
                          "(adapter: testscope):\n"
-                         "    unity::scopes::MiddlewareException: ObjectAdapter: broker thread failure "
-                         "(adapter: testscope): address in use: ipc://testscope", e.what());
+                         "    unity::scopes::MiddlewareException: safe_bind(): address in use: ipc://testscope",
+                         e.what());
         }
 
         {
@@ -1127,8 +1227,7 @@ TEST(ObjectAdapter, double_bind)
 
 TEST(ObjectAdapter, dflt_servant)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/zmq_middleware/ObjectAdapter/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    ZmqMiddleware mw("testscope", nullptr, zmq_ini);
 
     wait();
     ObjectAdapter a(mw, "testscope", "ipc://testscope", RequestMode::Twoway, 1);
@@ -1182,8 +1281,7 @@ TEST(ObjectAdapter, dflt_servant)
 
 TEST(ObjectAdapter, dflt_servant_exceptions)
 {
-    ZmqMiddleware mw("testscope", TEST_BUILD_ROOT "/gtest/scopes/internal/zmq_middleware/ObjectAdapter/Zmq.ini",
-                     (RuntimeImpl*)0x1);
+    ZmqMiddleware mw("testscope", nullptr, zmq_ini);
 
     wait();
     ObjectAdapter a(mw, "testscope", "ipc://testscope", RequestMode::Twoway, 1);
@@ -1212,7 +1310,13 @@ TEST(ObjectAdapter, dflt_servant_exceptions)
     }
     catch (MiddlewareException const& e)
     {
-        EXPECT_STREQ("unity::scopes::MiddlewareException: Object adapter in Failed state (adapter: testscope2)",
+        EXPECT_STREQ("unity::scopes::MiddlewareException: add_dflt_servant(): Object adapter in"
+                     " Failed state (adapter: testscope2)\n"
+                     "    Exception history:\n"
+                     "        Exception #1:\n"
+                     "            unity::scopes::MiddlewareException: ObjectAdapter: pump thread failure"
+                     " (adapter: testscope2):\n"
+                     "                unity::scopes::MiddlewareException: safe_bind(): address in use: ipc://testscope",
                      e.what());
     }
 
@@ -1224,7 +1328,13 @@ TEST(ObjectAdapter, dflt_servant_exceptions)
     }
     catch (MiddlewareException const& e)
     {
-        EXPECT_STREQ("unity::scopes::MiddlewareException: Object adapter in Failed state (adapter: testscope2)",
+        EXPECT_STREQ("unity::scopes::MiddlewareException: remove_dflt_servant(): Object adapter in"
+                     " Failed state (adapter: testscope2)\n"
+                     "    Exception history:\n"
+                     "        Exception #1:\n"
+                     "            unity::scopes::MiddlewareException: ObjectAdapter: pump thread failure"
+                     " (adapter: testscope2):\n"
+                     "                unity::scopes::MiddlewareException: safe_bind(): address in use: ipc://testscope",
                      e.what());
     }
 
@@ -1236,7 +1346,13 @@ TEST(ObjectAdapter, dflt_servant_exceptions)
     }
     catch (MiddlewareException const& e)
     {
-        EXPECT_STREQ("unity::scopes::MiddlewareException: Object adapter in Failed state (adapter: testscope2)",
+        EXPECT_STREQ("unity::scopes::MiddlewareException: find_dflt_servant(): Object adapter in"
+                     " Failed state (adapter: testscope2)\n"
+                     "    Exception history:\n"
+                     "        Exception #1:\n"
+                     "            unity::scopes::MiddlewareException: ObjectAdapter: pump thread failure"
+                     " (adapter: testscope2):\n"
+                     "                unity::scopes::MiddlewareException: safe_bind(): address in use: ipc://testscope",
                      e.what());
     }
 }
